@@ -30,7 +30,7 @@ namespace Konversation
 {
 
     ChannelOptionsDialog::ChannelOptionsDialog(Channel *channel)
-        : KDialogBase(channel, "channelOptions", false, i18n("Channel Options for %1").arg(channel->getName()), Ok|Cancel, Ok)
+        : KDialogBase(channel, "channelOptions", false, i18n("Channel Settings for %1").arg(channel->getName()), Ok|Cancel, Ok)
     {
         Q_ASSERT(channel);
         m_widget = new ChannelOptionsUI(this);
@@ -42,6 +42,7 @@ namespace Konversation
 
         // don't allow sorting. most recent topic is always first
         m_widget->topicHistoryList->setSortColumn(-1);
+        m_widget->banList->setDefaultRenameAction(QListView::Accept);
         // hide column where the complete topic will be put in for convenience
         m_widget->topicHistoryList->hideColumn(2);
         // do not allow the user to resize the hidden column back into view
@@ -60,10 +61,20 @@ namespace Konversation
         connect(m_channel, SIGNAL(modesChanged()), this, SLOT(refreshModes()));
         connect(m_channel->getOwnChannelNick(), SIGNAL(channelNickChanged()), this, SLOT(refreshEnableModes()));
 
-        connect(this, SIGNAL(cancelClicked()), this, SLOT(closeOptionsDialog()));
+        connect(this, SIGNAL(cancelClicked()), this, SLOT(hide()));
         connect(this, SIGNAL(okClicked()), this, SLOT(changeOptions()));
 
+        connect(m_channel, SIGNAL(banAdded(const QString&)), this, SLOT(addBan(const QString&)));
+        connect(m_channel, SIGNAL(banRemoved(const QString&)), this, SLOT(removeBan(const QString&)));
+        connect(m_channel, SIGNAL(banListCleared()), m_widget->banList, SLOT(clear()));
+
+        connect(m_widget->addBan, SIGNAL(clicked()), this, SLOT(addBanClicked()));
+        connect(m_widget->removeBan, SIGNAL(clicked()), this, SLOT(removeBanClicked()));
+        connect(m_widget->banList, SIGNAL(itemRenamed (QListViewItem*)), this, SLOT(banEdited(QListViewItem*)));
+        connect(m_widget->banList, SIGNAL(itemRenamed (QListViewItem*, int, const QString&)), this, SLOT(banEdited(QListViewItem*)));
+
         refreshTopicHistory();
+        refreshBanList();
         refreshAllowedChannelModes();
         refreshModes();
     }
@@ -72,22 +83,20 @@ namespace Konversation
     {
     }
 
-    void ChannelOptionsDialog::closeOptionsDialog()
-    {
-        deleteLater();
-    }
-
     void ChannelOptionsDialog::changeOptions()
     {
-        QString newTopic = topic();
+        QString newTopic = topic(), oldTopic=m_channel->getTopicHistory().first().section(' ', 2);
 
-        if(newTopic != m_channel->getTopicHistory().first().section(' ', 2))
+        if(newTopic != oldTopic)
         {
-            // Pass a \n to avoid whitespace stripping so we can determine if we want to clear the channel topic.
+            // Pass a ^A so we can determine if we want to clear the channel topic.
             if (newTopic.isEmpty())
-                m_channel->sendChannelText(Preferences::commandChar() + "TOPIC " + m_channel->getName() + " \n");
+            {
+                if (!oldTopic.isEmpty())
+                    m_channel->sendChannelText(Preferences::commandChar() + "TOPIC " + m_channel->getName() + " \x01");
+            }
             else
-                m_channel->sendChannelText(Preferences::commandChar() + "TOPIC " + m_channel->getName() + " " + newTopic);
+                m_channel->sendChannelText(Preferences::commandChar() + "TOPIC " + m_channel->getName() + ' ' + newTopic);
         }
 
         QStringList newModeList = modes();
@@ -103,7 +112,7 @@ namespace Konversation
         {
             modeString = (*it).mid(1);
             plus = ((*it)[0] == '+');
-            tmp = currentModeList.grep(QRegExp("^" + modeString));
+            tmp = currentModeList.grep(QRegExp('^' + modeString));
 
             if(tmp.isEmpty() && plus)
             {
@@ -111,10 +120,15 @@ namespace Konversation
             }
             else if(!tmp.isEmpty() && !plus)
             {
-                m_channel->getServer()->queue(command.arg(m_channel->getName()).arg("-").arg(modeString[0]).arg(modeString.mid(1)));
+                //FIXME: Bahamuth requires the key parameter for -k, but ircd breaks on -l with limit number.
+                //Hence two versions of this.
+                if (modeString[0] == 'k')
+                    m_channel->getServer()->queue(command.arg(m_channel->getName()).arg("-").arg(modeString[0]).arg(modeString.mid(1)));
+                else
+                    m_channel->getServer()->queue(command.arg(m_channel->getName()).arg("-").arg(modeString[0]).arg(""));
             }
         }
-        deleteLater();
+        hide();
     }
 
     void ChannelOptionsDialog::toggleAdvancedModes()
@@ -186,7 +200,12 @@ namespace Konversation
         m_widget->secretModeChBox->setEnabled(enable);
         m_widget->keyModeChBox->setEnabled(enable);
         m_widget->keyModeEdit->setEnabled(enable);
+
+        m_widget->banList->setItemsRenameable(enable);
+        m_widget->addBan->setEnabled(enable);
+        m_widget->removeBan->setEnabled(enable);
     }
+
     void ChannelOptionsDialog::refreshAllowedChannelModes()
     {
         QString modeString = m_channel->getServer()->allowedChannelModes();
@@ -299,27 +318,37 @@ namespace Konversation
         QString mode;
 
         mode = (m_widget->topicModeChBox->isChecked() ? "+" : "-");
-        mode += "t";
+        mode += 't';
         modes.append(mode);
         mode = (m_widget->messageModeChBox->isChecked() ? "+" : "-");
-        mode += "n";
+        mode += 'n';
         modes.append(mode);
         mode = (m_widget->userLimitChBox->isChecked() ? "+" : "-");
-        mode += "l" + m_widget->userLimitEdit->value();
+        mode += 'l' + QString::number( m_widget->userLimitEdit->value() );
         modes.append(mode);
         mode = (m_widget->inviteModeChBox->isChecked() ? "+" : "-");
-        mode += "i";
+        mode += 'i';
         modes.append(mode);
         mode = (m_widget->moderatedModeChBox->isChecked() ? "+" : "-");
-        mode += "m";
+        mode += 'm';
         modes.append(mode);
         mode = (m_widget->secretModeChBox->isChecked() ? "+" : "-");
-        mode += "s";
-        modes.append(mode);
-        mode = (m_widget->keyModeChBox->isChecked() ? "+" : "-");
-        mode += "k" + m_widget->keyModeEdit->text();
+        mode += 's';
         modes.append(mode);
 
+        if (m_widget->keyModeChBox->isChecked() && !m_widget->keyModeEdit->text().isEmpty())
+        {
+            mode = '+';
+            mode += 'k' + m_widget->keyModeEdit->text();
+            modes.append(mode);
+        }
+        else if (!m_widget->keyModeChBox->isChecked())
+        {
+            mode = '-';
+            mode += 'k' + m_widget->keyModeEdit->text();
+            modes.append(mode);
+        }
+
         QListViewItem* item = m_widget->otherModesList->firstChild();
 
         while(item)
@@ -333,6 +362,126 @@ namespace Konversation
         return modes;
     }
 
+    // Ban List tab related functions
+
+    void ChannelOptionsDialog::refreshBanList()
+    {
+        QStringList banlist = m_channel->getBanList();
+        m_widget->banList->clear();
+
+        for(QStringList::const_iterator it = banlist.fromLast(); it != banlist.end(); --it)
+        {
+            QDateTime date;
+            date.setTime_t((*it).section(' ', 2 ,2).toUInt());
+
+            new BanListViewItem(m_widget->banList, (*it).section(' ', 0, 0), (*it).section(' ', 1, 1).section('!', 0, 0), date.toString(Qt::LocalDate));
+        }
+    }
+
+    void ChannelOptionsDialog::addBan(const QString& newban)
+    {
+        QDateTime date;
+        date.setTime_t(newban.section(' ', 2 ,2).toUInt());
+
+        new BanListViewItem(m_widget->banList, newban.section(' ', 0, 0), newban.section(' ', 1, 1).section('!', 0, 0), date.toString(Qt::LocalDate));
+    }
+
+    void ChannelOptionsDialog::removeBan(const QString& ban)
+    {
+        delete m_widget->banList->findItem(ban, 0);
+    }
+
+    void ChannelOptionsDialog::banEdited(QListViewItem *edited)
+    {
+        if (edited == m_NewBan)
+        {
+            if (!m_NewBan->text(0).isEmpty())
+            {
+                m_channel->getServer()->requestBan(QStringList(m_NewBan->text(0)), m_channel->getName(), QString::null);
+            }
+
+            // We will delete the item and let the addBan slot handle
+            // readding the item because for some odd reason using
+            // startRename causes further attempts to rename the item
+            // using 2 mouse clicks to fail in odd ways.
+            delete edited;
+
+            return;
+        }
+
+        BanListViewItem *new_edited = dynamic_cast <BanListViewItem*> (edited);
+        if (new_edited == NULL) return; // Should not happen.
+
+        if (new_edited->getOldValue() != new_edited->text(0))
+        {
+            m_channel->getServer()->requestUnban(new_edited->getOldValue(), m_channel->getName());
+
+            if (!new_edited->text(0).isEmpty())
+            {
+                m_channel->getServer()->requestBan(QStringList(new_edited->text(0)), m_channel->getName(), QString::null);
+            }
+
+            // We delete the existing item because it's possible the server may
+            // Modify the ban causing us not to catch it. If that happens we'll be
+            // stuck with a stale item and a new item with the modified hostmask.
+            delete new_edited;
+        }
+    }
+
+    void ChannelOptionsDialog::addBanClicked()
+    {
+        m_NewBan = new BanListViewItem(m_widget->banList, true);
+
+        m_NewBan->setRenameEnabled(0,true);
+        m_NewBan->startRename(0);
+    }
+
+    void ChannelOptionsDialog::removeBanClicked()
+    {
+        if (m_widget->banList->currentItem())
+        {
+            m_channel->getServer()->requestUnban(m_widget->banList->currentItem()->text(0), m_channel->getName());
+        }
+    }
+
+    // This is our implementation of BanListViewItem
+
+    BanListViewItem::BanListViewItem(QListView *parent)
+      : KListViewItem(parent)
+    {
+        m_isNewBan = 0;
+    }
+
+    BanListViewItem::BanListViewItem(QListView *parent, bool isNew)
+      : KListViewItem(parent)
+    {
+        m_isNewBan = isNew;
+    }
+
+    BanListViewItem::BanListViewItem ( QListView *parent, QString label1, QString label2, QString label3, QString label4, QString label5, QString label6, QString label7, QString label8 ) : KListViewItem(parent, label1, label2, label3, label4, label5, label6, label7, label8)
+    {
+        m_isNewBan = 0;
+    }
+
+    BanListViewItem::BanListViewItem ( QListView *parent, bool isNew, QString label1, QString label2, QString label3, QString label4, QString label5, QString label6, QString label7, QString label8 ) : KListViewItem(parent, label1, label2, label3, label4, label5, label6, label7, label8)
+    {
+        m_isNewBan = isNew;
+    }
+
+    void BanListViewItem::startRename( int col )
+    {
+        m_oldValue = text(col);
+
+        KListViewItem::startRename(col);
+    }
+
+    void BanListViewItem::cancelRename( int col )
+    {
+        if (text(col).isEmpty() && m_isNewBan)
+            delete this;
+        else
+            KListViewItem::cancelRename(col);
+    }
 }
 
 #include "channeloptionsdialog.moc"
