@@ -30,7 +30,6 @@
 #include "ircview.h"
 #include "server.h"
 #include "konversationapplication.h"
-#include "konversationmainwindow.h"
 #include "logfilereader.h"
 
 ChatWindow::ChatWindow(QWidget* parent) : QVBox(parent)
@@ -41,15 +40,26 @@ ChatWindow::ChatWindow(QWidget* parent) : QVBox(parent)
     m_server=0;
     m_notificationsEnabled = true;
     m_channelEncodingSupported = false;
-    m_mainWindow=NULL;
     m_currentTabNotify = Konversation::tnfNone;
 
     setMargin(margin());
     setSpacing(spacing());
+
+    // The font size of the KTabWidget container may be inappropriately
+    // small due to the "Tab bar" font size setting.
+    setFont(KGlobalSettings::generalFont());
 }
 
 ChatWindow::~ChatWindow()
 {
+    emit closing(this);
+}
+
+void ChatWindow::updateAppearance()
+{
+    // The font size of the KTabWidget container may be inappropriately
+    // small due to the "Tab bar" font size setting.
+    setFont(KGlobalSettings::generalFont());
 }
 
 void ChatWindow::setName(const QString& newName)
@@ -75,14 +85,13 @@ ChatWindow::WindowType ChatWindow::getType()
 
 void ChatWindow::setServer(Server* newServer)
 {
-    if(!newServer)
-      { 
-	kdDebug("ChatWindow::setServer(0)!") << endl;
-      }
+    if (!newServer)
+    {
+        kdDebug("ChatWindow::setServer(0)!") << endl;
+    }
     else
     {
         m_server=newServer;
-        setMainWindow(m_server->getMainWindow());
         connect(m_server,SIGNAL (serverOnline(bool)),this,SLOT (serverOnline(bool)) );
 
         // check if we need to set up the signals
@@ -96,11 +105,6 @@ void ChatWindow::setServer(Server* newServer)
     }
 }
 
-void ChatWindow::setMainWindow(KonversationMainWindow *mainWindow)
-{
-    m_mainWindow = mainWindow;
-}
-
 Server* ChatWindow::getServer()
 {
     return m_server;
@@ -127,16 +131,15 @@ void ChatWindow::setTextView(IRCView* newView)
     }
 
     textView->setChatWin(this);
-    connect(textView,SIGNAL (textToLog(const QString&)),this,SLOT (logText(const QString&)) );
-    connect(textView,SIGNAL (actionStatusText( const QString & ) ),this, SIGNAL (actionStatusText( const QString & ) ));
-    connect(textView,SIGNAL (clearStatusText()), this, SIGNAL (clearStatusText()));
+    connect(textView,SIGNAL(textToLog(const QString&)), this,SLOT(logText(const QString&)));
+    connect(textView,SIGNAL(setStatusBarTempText(const QString&)), this, SIGNAL(setStatusBarTempText(const QString&)));
+    connect(textView,SIGNAL(clearStatusBarTempText()), this, SIGNAL(clearStatusBarTempText()));
 }
 
 void ChatWindow::insertRememberLine()
 {
     if(!textView) return;
-    kdDebug() << "Inserting remember line" << endl;
-    textView->appendRaw("<br><hr color=\""+Preferences::color(Preferences::CommandMessage).name()+"\" noshade>", true, true);
+    textView->appendLine();
 }
 
 void ChatWindow::appendRaw(const QString& message, bool suppressTimestamps)
@@ -163,10 +166,10 @@ void ChatWindow::appendAction(const QString& nickname,const QString& message, bo
     textView->appendAction(nickname,message);
 }
 
-void ChatWindow::appendServerMessage(const QString& type,const QString& message)
+void ChatWindow::appendServerMessage(const QString& type,const QString& message, bool parseURL)
 {
     if(!textView) return ;
-    textView->appendServerMessage(type,message);
+    textView->appendServerMessage(type,message, parseURL);
 }
 
 void ChatWindow::appendCommandMessage(const QString& command,const QString& message, bool important, bool parseURL, bool self)
@@ -197,7 +200,7 @@ void ChatWindow::cdIntoLogPath()
     }
 
     // add the logfile name to the path
-    logfile.setName(logPath.path()+"/"+logName);
+    logfile.setName(logPath.path()+'/'+logName);
 }
 
 void ChatWindow::setLogfileName(const QString& name)
@@ -206,7 +209,7 @@ void ChatWindow::setLogfileName(const QString& name)
     if(firstLog)
     {
         // status panels get special treatment here, since they have no server at the beginning
-        if(getType() == Status)
+        if (getType() == Status || getType() == DccChat)
         {
             logName = name + ".log";
         }
@@ -216,52 +219,101 @@ void ChatWindow::setLogfileName(const QString& name)
             logName = QString(m_server->getServerGroup().lower()).append('_').append(name).append(".log").replace('/','_');
         }
 
-        // "cd" into log path or create path, if it's not there
-        cdIntoLogPath();
-        // Show last log lines. This idea was stole ... um ... inspired by PMP :)
-        // Don't do this for the server status windows, though
-        if((getType() != Status) && logfile.open(IO_ReadOnly))
+        // load backlog to show
+        if(Preferences::showBacklog())
         {
-            unsigned long filePosition;
-
-            QString backlogLine;
-            QTextStream backlog(&logfile);
-            backlog.setEncoding(QTextStream::UnicodeUTF8);
-            // Check if the log is actually big enough
-            if(backlog.device()->size() > 1024)
+            // "cd" into log path or create path, if it's not there
+            cdIntoLogPath();
+            // Show last log lines. This idea was stole ... um ... inspired by PMP :)
+            // Don't do this for the server status windows, though
+            if((getType() != Status) && logfile.open(IO_ReadOnly))
             {
-                // Set file pointer to 1 kB from the end
-                backlog.device()->at(backlog.device()->size() - 1024);
-                // Skip first line, since it may be incomplete
-                backlog.readLine();
-            }
+                unsigned long filePosition;
 
-            // Loop until end of file reached
-            while(!backlog.atEnd())
-            {
-                // remember actual file position to check for deadlocks
-                filePosition = backlog.device()->at();
-                backlogLine = backlog.readLine();
+                QString backlogLine;
+                QTextStream backlog(&logfile);
+                backlog.setEncoding(QTextStream::UnicodeUTF8);
 
-                // check for deadlocks
-                if(backlog.device()->at() == filePosition) backlog.device()->at(filePosition + 1);
-
-                // if a tab character is present in the line
-                if(backlogLine.find('\t') != -1)
+                QStringList firstColumns;
+                QStringList messages;
+                int offset = 0;
+                unsigned int lastPacketHeadPosition = backlog.device()->size();
+                const unsigned int packetSize = 4096;
+                while(messages.count() < (unsigned int)Preferences::backlogLines() && backlog.device()->size() > packetSize * offset)
                 {
-                    // extract first column from log
-                    QString backlogFirst = backlogLine.left(backlogLine.find('\t'));
-                    // cut first column from line
-                    backlogLine = backlogLine.mid(backlogLine.find('\t') + 1);
-                    // Logfile is in utf8 so we don't need to do encoding stuff here
-                    // append backlog with time and first column to text view
-                    appendBacklogMessage(backlogFirst, backlogLine);
-                }
-            } // while
+                    QStringList firstColumnsInPacket;
+                    QStringList messagesInPacket;
 
-            backlog.unsetDevice();
-            logfile.close();
-        }
+                    // packetSize * offset < size <= packetSize * ( offset + 1 )
+
+                    // Check if the log is bigger than packetSize * ( offset + 1 )
+                    if(backlog.device()->size() > packetSize * ( offset + 1 ))
+                    {
+                        // Set file pointer to the packet size above the offset
+                        backlog.device()->at(backlog.device()->size() - packetSize * ( offset + 1 ));
+                        // Skip first line, since it may be incomplete
+                        backlog.readLine();
+                    }
+                    else
+                    {
+                        // Set file pointer to the head
+                        backlog.device()->reset();
+                    }
+
+                    unsigned int currentPacketHeadPosition = backlog.device()->at();
+
+                    // Loop until end of file reached
+                    while(!backlog.atEnd() && backlog.device()->at() < lastPacketHeadPosition)
+                    {
+                        // remember actual file position to check for deadlocks
+                        filePosition = backlog.device()->at();
+                        backlogLine = backlog.readLine();
+
+                        // check for deadlocks
+                        if(backlog.device()->at() == filePosition) backlog.device()->at(filePosition + 1);
+
+                        // if a tab character is present in the line
+                        if(backlogLine.find('\t') != -1)
+                        {
+                            // extract first column from log
+                            QString backlogFirst = backlogLine.left(backlogLine.find('\t'));
+                            // cut first column from line
+                            backlogLine = backlogLine.mid(backlogLine.find('\t') + 1);
+                            // Logfile is in utf8 so we don't need to do encoding stuff here
+                            // append backlog with time and first column to text view
+                            firstColumnsInPacket << backlogFirst;
+                            messagesInPacket << backlogLine;
+                        }
+                    } // while
+
+                    // remember the position not to read the same lines again
+                    lastPacketHeadPosition = currentPacketHeadPosition;
+                    ++offset;
+
+                    firstColumns = firstColumnsInPacket + firstColumns;
+                    messages = messagesInPacket + messages;
+                }
+                backlog.unsetDevice();
+                logfile.close();
+
+                // trim
+                int surplus = messages.count() - Preferences::backlogLines();
+                // "surplus" can be a minus value. (when the backlog is too short)
+                if(surplus > 0)
+                {
+                    for(int i = 0 ; i < surplus ; ++i)
+                    {
+                        firstColumns.pop_front();
+                        messages.pop_front();
+                    }
+                }
+
+                QStringList::Iterator itFirstColumn = firstColumns.begin();
+                QStringList::Iterator itMessage = messages.begin();
+                for( ; itFirstColumn != firstColumns.end() ; ++itFirstColumn, ++itMessage )
+                    appendBacklogMessage(*itFirstColumn, *itMessage);
+            }
+        } // if(Preferences::showBacklog())
     }
 }
 
@@ -377,7 +429,9 @@ bool ChatWindow::eventFilter(QObject* watched, QEvent* e)
     {
         QKeyEvent* ke = static_cast<QKeyEvent*>(e);
 
-        if(ke->key() == Qt::Key_Up && ke->state() == Qt::ShiftButton)
+        bool scrollMod = (Preferences::useMultiRowInputBox() ? false : (ke->state() == Qt::ShiftButton));
+
+        if(ke->key() == Qt::Key_Up && scrollMod)
         {
             if(textView)
             {
@@ -387,7 +441,7 @@ bool ChatWindow::eventFilter(QObject* watched, QEvent* e)
 
             return true;
         }
-        else if(ke->key() == Qt::Key_Down && ke->state() == Qt::ShiftButton)
+        else if(ke->key() == Qt::Key_Down && scrollMod)
         {
             if(textView)
             {
