@@ -21,6 +21,7 @@
 #include <kdeversion.h>
 #include <kdialog.h>
 #include <kglobal.h>
+#include <kglobalsettings.h>
 #include <kiconloader.h>
 #include <klistview.h>
 #include <klocale.h>
@@ -37,6 +38,7 @@ DccPanel::DccPanel(QWidget* parent)
 : ChatWindow(parent)
 {
     setType(ChatWindow::DccPanel);
+    setName(i18n("DCC Status"));
 
     m_listView = new KListView(this,"dcc_control_panel");
 
@@ -50,18 +52,18 @@ DccPanel::DccPanel(QWidget* parent)
         m_listView->addColumn("");
 
     //m_listView->setColumnText(Column::TypeIcon,      "");
-    m_listView->setColumnText(Column::OfferDate,     i18n("Date"));
+    m_listView->setColumnText(Column::OfferDate,     i18n("Started at"));
     m_listView->setColumnText(Column::Status,        i18n("Status"));
     m_listView->setColumnText(Column::FileName,      i18n("File"));
     m_listView->setColumnText(Column::PartnerNick,   i18n("Partner"));
     m_listView->setColumnText(Column::Progress,      i18n("Progress"));
     m_listView->setColumnText(Column::Position,      i18n("Position"));
-    m_listView->setColumnText(Column::TimeRemaining, i18n("Remain"));
+    m_listView->setColumnText(Column::TimeRemaining, i18n("Remaining"));
     m_listView->setColumnText(Column::CPS,           i18n("Speed"));
     m_listView->setColumnText(Column::SenderAddress, i18n("Sender Address"));
 
     m_listView->setColumnWidth(Column::TypeIcon,       16);
-    m_listView->setColumnWidth(Column::OfferDate,      70);
+    m_listView->setColumnWidth(Column::OfferDate,      90);
     m_listView->setColumnWidth(Column::Status,         80);
     m_listView->setColumnWidth(Column::FileName,      150);
     m_listView->setColumnWidth(Column::PartnerNick,    70);
@@ -76,8 +78,8 @@ DccPanel::DccPanel(QWidget* parent)
     m_listView->setColumnAlignment(Column::OfferDate,     AlignHCenter);
     m_listView->setColumnAlignment(Column::Progress,      AlignHCenter);
     m_listView->setColumnAlignment(Column::Position,      AlignHCenter);
-    m_listView->setColumnAlignment(Column::TimeRemaining, AlignRight);
-    m_listView->setColumnAlignment(Column::CPS,           AlignRight);
+    m_listView->setColumnAlignment(Column::TimeRemaining, AlignHCenter);
+    m_listView->setColumnAlignment(Column::CPS,           AlignHCenter);
 
     m_listView->setSorting(Column::OfferDate, false);
 
@@ -91,19 +93,19 @@ DccPanel::DccPanel(QWidget* parent)
     // convenience, undeffed below again to avoid name clashes
     #define icon(s) KGlobal::iconLoader()->loadIconSet( s, KIcon::Small )
 
-    m_buttonAccept = new QPushButton(icon("player_play"), i18n("&Accept"), buttonsBox, "start_dcc");
-    m_buttonAbort  = new QPushButton(icon("stop"),        i18n("A&bort"),  buttonsBox, "abort_dcc");
-    m_buttonClear  = new QPushButton(icon("editdelete"),  i18n("&Clear"),  buttonsBox, "clear_dcc");
-    m_buttonOpen   = new QPushButton(icon("exec"),        i18n("&Open File"),   buttonsBox, "open_dcc_file");
-    m_buttonRemove = new QPushButton(icon("edittrash"),   i18n("&Remove File"), buttonsBox, "remove_dcc_file");
-    m_buttonDetail = new QPushButton(icon("view_text"),   i18n("DCC &Detail"), buttonsBox, "detail_dcc");
+    m_buttonAccept = new QPushButton(icon("player_play"), i18n("Accept"), buttonsBox, "start_dcc");
+    m_buttonAbort  = new QPushButton(icon("stop"),        i18n("Abort"),  buttonsBox, "abort_dcc");
+    m_buttonClear  = new QPushButton(icon("editdelete"),  i18n("Clear"),  buttonsBox, "clear_dcc");
+    m_buttonOpen   = new QPushButton(icon("exec"),        i18n("Open File"),   buttonsBox, "open_dcc_file");
+    m_buttonRemove = new QPushButton(icon("edittrash"),   i18n("Remove File"), buttonsBox, "remove_dcc_file");
+    m_buttonDetail = new QPushButton(icon("view_text"),   i18n("Transfer Details"), buttonsBox, "detail_dcc");
 
     QToolTip::add( m_buttonAccept, i18n( "Start receiving" ) );
     QToolTip::add( m_buttonAbort,  i18n( "Abort the transfer(s)" ) );
     QToolTip::add( m_buttonClear,  i18n( "Remove from this panel" ) );
     QToolTip::add( m_buttonOpen,   i18n( "Run the file" ) );
     QToolTip::add( m_buttonRemove, i18n( "Remove the received file(s)" ) );
-    QToolTip::add( m_buttonDetail, i18n( "View DCC detail information" ) );
+    QToolTip::add( m_buttonDetail, i18n( "View DCC transfer details" ) );
 
     connect( m_buttonAccept, SIGNAL(clicked()), this, SLOT(acceptDcc()) );
     connect( m_buttonAbort,  SIGNAL(clicked()), this, SLOT(abortDcc()) );
@@ -125,9 +127,10 @@ DccPanel::DccPanel(QWidget* parent)
     m_popup->insertSeparator();                   // -----
     m_popup->insertItem(icon("exec"),            i18n("&Open File"),                  Popup::Open);
     m_popup->insertItem(icon("edittrash"),       i18n("&Remove File"),                Popup::Remove);
-    m_popup->insertItem(icon("messagebox_info"), i18n("File &Information"),           Popup::Info);
+//FIXME: Disabled for 1.0 release, as it doesn't appear to work at all.
+//    m_popup->insertItem(icon("messagebox_info"), i18n("File &Information"),           Popup::Info);
     m_popup->insertSeparator();                   // -----
-    m_popup->insertItem(icon("view_text"),       i18n("DCC &Detail Information"),     Popup::Detail);
+    m_popup->insertItem(icon("view_text"),       i18n("DCC Transfer &Details"),     Popup::Detail);
 
     #undef icon
 
@@ -483,6 +486,29 @@ DccTransfer* DccPanel::getTransferByName(const QString& name,DccTransfer::DccTyp
     return 0;
 }
 
+bool DccPanel::isLocalFileInWritingProcess( const KURL& url )
+{
+    int index=0;
+    DccTransfer* item;
+    do
+    {
+        // TODO: Get rid of this cast
+        item=static_cast<DccTransfer*>(getListView()->itemAtIndex(index++));
+        if(item)
+        {
+            if( item->getType() == DccTransfer::Receive &&
+                ( item->getStatus() == DccTransfer::Connecting ||
+                  item->getStatus() == DccTransfer::Receiving ) &&
+                item->getFileURL() == url )
+            {
+                return true;
+            }
+        }
+    } while(item);
+
+    return false;
+}
+
 // virtual
 void DccPanel::childAdjustFocus()
 {
