@@ -37,7 +37,8 @@
 #define TIME_REMAINING_NOT_AVAILABLE -1
 #define TIME_REMAINING_INFINITE      -2
 
-#define CPS_UNKNOWN -1
+#define CPS_CALCULATING -1
+#define CPS_NOT_IN_TRANSFER -2
 
 DccTransfer::DccTransfer( DccPanel* panel, DccType dccType, const QString& partnerNick )
 : KListViewItem( panel->getListView() )
@@ -49,10 +50,6 @@ DccTransfer::DccTransfer( DccPanel* panel, DccType dccType, const QString& partn
     m_resumed = false;
     m_transferringPosition = 0;
     m_transferStartPosition = 0;
-    m_partnerIp = QString::null;
-    m_partnerPort = QString::null;
-    m_ownIp = QString::null;
-    m_ownPort = QString::null;
     m_timeOffer = QDateTime::currentDateTime();
 
     m_bufferSize = Preferences::dccBufferSize();
@@ -67,9 +64,7 @@ DccTransfer::DccTransfer( DccPanel* panel, DccType dccType, const QString& partn
 
     m_detailDialog = 0;
 
-    // FIXME: we shouldn't do these init in the instance constructer
-    s_dccTypeText[ Send ]    = i18n("Send");
-    s_dccTypeText[ Receive ] = i18n("Receive");
+    m_panel = panel;
 
     s_dccStatusText[ Queued ]        = i18n("Queued");
     s_dccStatusText[ Preparing ]     = i18n("Preparing");
@@ -110,7 +105,10 @@ void DccTransfer::updateView()
     if ( m_fileSize )
         m_progressBar->setProgress( getProgress() );
     else // filesize is unknown
+    {
+        m_progressBar->hide();
         setText( DccPanel::Column::Progress, i18n( "unknown" ) );
+    }
 
     if ( m_detailDialog )
         m_detailDialog->updateView();
@@ -206,7 +204,6 @@ void DccTransfer::slotLogTransfer()
     m_transferLogPosition.append( m_transferringPosition );
 }
 
-                                    
 void DccTransfer::paintCell( QPainter* painter, const QColorGroup& colorgroup, int column, int width, int alignment ) // virtual public
 {
     KListViewItem::paintCell( painter, colorgroup, column, width, alignment );
@@ -288,7 +285,7 @@ void DccTransfer::openFileInfoDialog()
                         if(item.isValid())
                         {
                             // append item information to list
-                            infoList.append("- "+item.translatedKey()+" "+item.string());
+                            infoList.append("- "+item.translatedKey()+' '+item.string());
                         }
                     } // endfor
                 }
@@ -339,10 +336,11 @@ void DccTransfer::closeDetailDialog()
 void DccTransfer::setStatus( DccStatus status, const QString& statusDetail )
 {
     bool changed = ( status != m_dccStatus );
+    DccStatus oldStatus = m_dccStatus;
     m_dccStatus = status;
     m_dccStatusDetail = statusDetail;
     if ( changed )
-        emit statusChanged( this );
+        emit statusChanged( this, m_dccStatus, oldStatus );
 }
 
 void DccTransfer::updateTransferMeters()
@@ -372,7 +370,7 @@ void DccTransfer::updateTransferMeters()
         if ( m_transferLogTime.count() >= 2 )
             m_cps = (double)( m_transferLogPosition.last() - m_transferLogPosition.front() ) / (double)( m_transferLogTime.last() - m_transferLogTime.front() ) * 1000;
         else // avoid zero devision
-            m_cps = CPS_UNKNOWN;
+            m_cps = CPS_CALCULATING;
 
         // update the remaining time
         if ( m_cps <= 0 )
@@ -384,7 +382,7 @@ void DccTransfer::updateTransferMeters()
     {
         // avoid zero devision
         if ( m_timeTransferStarted.secsTo( m_timeTransferFinished ) <= 0 )
-            m_cps = CPS_UNKNOWN;
+            m_cps = CPS_NOT_IN_TRANSFER;
         else
             m_cps = (double)( m_transferringPosition - m_transferStartPosition ) / (double)m_timeTransferStarted.secsTo( m_timeTransferFinished );
         m_timeRemaining = TIME_REMAINING_NOT_AVAILABLE;
@@ -396,21 +394,6 @@ void DccTransfer::updateTransferMeters()
     }
 }
 
-QString DccTransfer::getTypeText() const
-{
-    return s_dccTypeText[ m_dccType ];
-}
-
-QPixmap DccTransfer::getTypeIcon() const
-{
-    QString icon;
-    if ( m_dccType == Send )
-        icon = "up";
-    else if ( m_dccType == Receive )
-        icon = "down";
-    return KGlobal::iconLoader()->loadIcon( icon, KIcon::Small );
-}
-
 QPixmap DccTransfer::getStatusIcon() const
 {
     QString icon;
@@ -462,7 +445,8 @@ int DccTransfer::getProgress() const
 QString DccTransfer::getPositionPrettyText( bool detailed ) const
 {
     if ( detailed )
-        return getPrettyNumberText( QString::number( m_transferringPosition ) ) + " / " + getPrettyNumberText( QString::number( m_fileSize ) );
+        return KGlobal::locale()->formatNumber( m_transferringPosition, 0 )  + " / " +
+            KGlobal::locale()->formatNumber( m_fileSize, 0 );
     else
         return KIO::convertSize( m_transferringPosition ) + " / " + KIO::convertSize( m_fileSize );
 }
@@ -493,8 +477,10 @@ QString DccTransfer::getTimeRemainingPrettyText() const
 
 QString DccTransfer::getCPSPrettyText() const
 {
-    if ( m_cps == CPS_UNKNOWN )
+    if ( m_cps == CPS_CALCULATING )
         return QString( "?" );
+    else if ( m_cps == CPS_NOT_IN_TRANSFER )
+        return QString();
     else
         return i18n("%1/sec").arg( KIO::convertSize( (KIO::fileoffset_t)m_cps ) );
 }
@@ -526,16 +512,7 @@ unsigned long DccTransfer::intel( unsigned long value )
     return value;
 }
 
-QString DccTransfer::getPrettyNumberText( const QString& numberText )
-{
-    QString prettyNumberText = numberText;
-    int commas = (int)( ( numberText.length() - 1 ) / 3 );
-    for ( int i=0 ; i < commas ; ++i )
-        prettyNumberText.insert( numberText.length() - ( ( i + 1 ) * 3 ), "," );
-    return prettyNumberText;
-}
-
-DccTransfer::DccType DccTransfer::getType() const 
+DccTransfer::DccType DccTransfer::getType() const
 { 
   return m_dccType; 
 }
@@ -545,6 +522,11 @@ DccTransfer::DccStatus DccTransfer::getStatus() const
   return m_dccStatus; 
 }
 
+const QString& DccTransfer::getStatusDetail() const
+{
+    return m_dccStatusDetail;
+}
+
 QDateTime DccTransfer::getTimeOffer() const 
 {
   return m_timeOffer; 
@@ -610,7 +592,6 @@ int DccTransfer::getTimeRemaining() const
   return m_timeRemaining; 
 }
 
-QString DccTransfer::s_dccTypeText[ DccTransfer::DccTypeCount ];
 QString DccTransfer::s_dccStatusText[ DccTransfer::DccStatusCount ];
 
 #include "dcctransfer.moc"
