@@ -63,7 +63,7 @@ class DccTransferRecv : public DccTransfer
          */
         virtual void start();
         /** The user has chosen to abort.
-         *  Either by chosen to abort directly, or by chosing cancel when
+         *  Either by chosen to abort directly, or by choosing cancel when
          *  prompted for information on where to save etc.
          *  Not called when it fails due to another problem.
          */
@@ -123,6 +123,10 @@ class DccTransferRecv : public DccTransfer
         bool m_partialFileExists;
         QTimer* m_connectionTimer;
         KNetwork::KStreamSocket* m_recvSocket;
+
+    private:
+        virtual QString getTypeText() const;
+        virtual QPixmap getTypeIcon() const;
 };
 
 class DccTransferRecvWriteCacheHandler : public QObject
