@@ -26,6 +26,7 @@
 #include <qtimer.h>
 
 #include <kdebug.h>
+#include <kiconloader.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kserversocket.h>
@@ -99,10 +100,12 @@ DccTransferSend::DccTransferSend( DccPanel* panel, const QString& partnerNick, c
         return;
     }
 
+    //FIXME: KIO::NetAccess::download() is a synchronous function. we should use KIO::get() instead.
     //Download the file.  Does nothing if it's local (file:/)
     if ( !KIO::NetAccess::download( m_fileURL, m_tmpFile, listView() ) )
     {
         failed( i18n( "Could not retrieve \"%1\"" ).arg( m_fileURL.prettyURL() ) );
+        kdDebug() << "DccTransferSend::DccTransferSend(): KIO::NetAccess::download() failed. reason: " << KIO::NetAccess::lastErrorString() << endl;
         return;
     }
 
@@ -119,8 +122,12 @@ DccTransferSend::DccTransferSend( DccPanel* panel, const QString& partnerNick, c
         }
     }
 
-    m_fileName = m_fileName.lower();
-    m_fileName.replace( " ", "_" );
+    if (Preferences::dccSpaceToUnderscore())
+        m_fileName.replace( " ", "_" );
+    else {
+        if (m_fileName.contains(" ") > 0)
+            m_fileName = "\"" + m_fileName + "\"";
+    }
 
     m_file.setName( m_tmpFile );
 
@@ -139,6 +146,16 @@ DccTransferSend::~DccTransferSend()
     cleanUp();
 }
 
+QString DccTransferSend::getTypeText() const
+{
+    return i18n( "Send" );
+}
+
+QPixmap DccTransferSend::getTypeIcon() const
+{
+    return KGlobal::iconLoader()->loadIcon( "up", KIcon::Small );
+}
+
 void DccTransferSend::cleanUp()
 {
     kdDebug() << "DccTransferSend::cleanUp()" << endl;
@@ -166,7 +183,7 @@ void DccTransferSend::failed( const QString& errorMessage )
     setStatus( Failed, errorMessage );
     updateView();
     cleanUp();
-    emit done( m_fileURL.path(), Failed, errorMessage );
+    emit done( this );
     openDetailDialog();
 }
 
@@ -177,7 +194,7 @@ void DccTransferSend::abort()                     // public slot
     setStatus( Aborted );
     updateView();
     cleanUp();
-    emit done( m_fileURL.path(), Aborted );
+    emit done( this );
 }
 
 void DccTransferSend::start()                     // public slot
@@ -191,7 +208,7 @@ void DccTransferSend::start()                     // public slot
     m_serverSocket = new KNetwork::KServerSocket( this );
     m_serverSocket->setFamily( KNetwork::KResolver::InetFamily );
 
-                                                  // user is specifing ports
+                                                  // user is specifying ports
     if ( Preferences::dccSpecificSendPorts() )
     {
         // set port
@@ -212,7 +229,7 @@ void DccTransferSend::start()                     // public slot
             return;
         }
     }
-    else                                          // user isn't specifing ports
+    else                                          // user isn't specifying ports
     {
         // Let the operating system choose a port
         m_serverSocket->setAddress( "0" );
@@ -348,7 +365,7 @@ void DccTransferSend::getAck()                    // slot
             setStatus( Done );
             updateView();
             cleanUp();
-            emit done( m_fileURL.path() );
+            emit done( this );
             break;                                // for safe
         }
     }
@@ -406,16 +423,16 @@ QString DccTransferSend::getQFileErrorString( int code )
             errorString=i18n("The operation was successful. Should never happen in an error dialog.");
             break;
         case IO_ReadError:
-            errorString=i18n("Could not read from file \"%1\".");
+            errorString=i18n("Could not read from file \"%1\".").arg( m_fileName );
             break;
         case IO_WriteError:
-            errorString=i18n("Could not write to file \"%1\".");
+            errorString=i18n("Could not write to file \"%1\".").arg( m_fileName );
             break;
         case IO_FatalError:
             errorString=i18n("A fatal unrecoverable error occurred.");
             break;
         case IO_OpenError:
-            errorString=i18n("Could not open file \"%1\".");
+            errorString=i18n("Could not open file \"%1\".").arg( m_fileName );
             break;
 
             // Same case value? Damn!
