@@ -29,6 +29,7 @@
 #include <qmap.h>
 #include <qcolor.h>
 #include <qscrollbar.h>
+#include <qcursor.h>
 
 #include <dcopref.h>
 #include <dcopclient.h>
@@ -48,18 +49,18 @@
 #include <kaction.h>
 #include <kglobalsettings.h>
 #include <kdebug.h>
+#include <kmenubar.h>
 
 #include "channel.h"
+#include "dccchat.h"
 #include "konversationapplication.h"
 #include "konversationmainwindow.h"
+#include "viewcontainer.h"
 #include "ircview.h"
 #include "highlight.h"
 #include "server.h"
-#include "searchdialog.h"
 #include "konversationsound.h"
-#include "chatwindow.h"
 #include "common.h"
-#include "images.h"
 #include "emoticon.h"
 #include "notificationhandler.h"
 
@@ -69,12 +70,12 @@ IRCView::IRCView(QWidget* parent, Server* newServer) : KTextBrowser(parent)
     m_resetScrollbar = true;
     m_offset = 0;
     m_mousePressed = false;
-    m_currentNick = QString::null;
     m_isOnNick = false;
     m_isOnChannel = false;
     m_chatWin = 0;
     m_findParagraph=0;
     m_findIndex=0;
+    m_lastInsertionWasLine = false;
 
     setAutoFormatting(QTextEdit::AutoNone);
     setUndoRedoEnabled(0);
@@ -91,18 +92,23 @@ IRCView::IRCView(QWidget* parent, Server* newServer) : KTextBrowser(parent)
     setStyleSheet(sheet);
 
     m_popup = new QPopupMenu(this,"ircview_context_menu");
+    toggleMenuBarSeparator = m_popup->insertSeparator();
+    m_popup->setItemVisible(toggleMenuBarSeparator, false);
     m_popup->insertItem(SmallIconSet("editcopy"),i18n("&Copy"),Copy);
     m_popup->insertItem(i18n("Select All"),SelectAll);
     m_popup->insertSeparator();
     m_popup->insertItem(SmallIcon("find"),i18n("Find Text..."),Search);
-    m_popup->insertSeparator();
 
     setServer(newServer);
-    setFont(Preferences::textFont());
+
     setViewBackground(Preferences::color(Preferences::TextViewBackground),QString::null);
 
+    if (Preferences::customTextFont())
+        setFont(Preferences::textFont());
+    else
+        setFont(KGlobalSettings::generalFont());
+
     connect(this, SIGNAL(highlighted(const QString&)), this, SLOT(highlightedSlot(const QString&)));
-    connect(this, SIGNAL(linkClicked(const QString&)), this, SLOT(urlClickSlot(const QString&)));
 }
 
 IRCView::~IRCView()
@@ -159,11 +165,14 @@ void IRCView::setViewBackground(const QColor& backgroundColor, const QString& pi
 void IRCView::setServer(Server* newServer)
 {
     m_server = newServer;
-    if(newServer) {
-      KAction *action = newServer->getMainWindow()->actionCollection()->action("open_logfile");
-      Q_ASSERT(action);
-      if(!action) return;
-      action->plug(m_popup);
+
+    if (newServer)
+    {
+        KAction *action = newServer->getViewContainer()->actionCollection()->action("open_logfile");
+        Q_ASSERT(action);
+        if(!action) return;
+        m_popup->insertSeparator();
+        action->plug(m_popup);
     }
 
 }
@@ -184,22 +193,26 @@ void IRCView::clear()
     KTextBrowser::clear();
 }
 
-void IRCView::highlightedSlot(const QString& link)
+void IRCView::highlightedSlot(const QString& _link)
 {
-    if(link == m_lastStatusText && !link.isEmpty()) {
-        //we just saw this a second ago.  no need to reemit.
+    //Hack to handle the fact that we get a decoded url
+    QString link = KURL::fromPathOrURL(_link).url();
+    //we just saw this a second ago.  no need to reemit.
+    if (link == m_lastStatusText && !link.isEmpty())
         return;
-    }
 
     // remember current URL to overcome link clicking problems in QTextBrowser
-    m_highlightedURL=link;
+    m_highlightedURL = link;
 
-    if(link.isEmpty()) {
-        if(!m_lastStatusText.isEmpty()) {
-          emit clearStatusText();
-          m_lastStatusText = QString::null;
+    if (link.isEmpty())
+    {
+        if (!m_lastStatusText.isEmpty()) 
+        {
+            emit clearStatusBarTempText();
+            m_lastStatusText = QString::null;
         }
-    } else {
+    } else
+    {
         m_lastStatusText = link;
     }
 
@@ -208,17 +221,17 @@ void IRCView::highlightedSlot(const QString& link)
         m_isOnNick = false;
         m_isOnChannel = false;
 
-        if(!link.isEmpty()) {
+        if (!link.isEmpty()) {
             //link therefore != m_lastStatusText  so emit with this new text
-            emit actionStatusText(link);
+            emit setStatusBarTempText(link);
         }
-        if(link.isEmpty() && m_copyUrlMenu)
+        if (link.isEmpty() && m_copyUrlMenu)
         {
             m_popup->removeItem(CopyUrl);
             m_popup->removeItem(Bookmark);
             m_copyUrlMenu = false;
         }
-        else if(!link.isEmpty() && !m_copyUrlMenu)
+        else if (!link.isEmpty() && !m_copyUrlMenu)
         {
             m_popup->insertItem(i18n("Copy URL to Clipboard"),CopyUrl,1);
             m_popup->insertItem(i18n("Add to Bookmarks"),Bookmark,2);
@@ -226,14 +239,16 @@ void IRCView::highlightedSlot(const QString& link)
             m_urlToCopy = link;
         }
     }
-    else if(link.startsWith("#") && !link.startsWith("##"))
+    else if (link.startsWith("#") && !link.startsWith("##"))
     {
         m_currentNick = link.mid(1);
         m_nickPopup->changeTitle(m_nickPopupId,m_currentNick);
         m_isOnNick = true;
-        emit actionStatusText( i18n("Open a query with %1").arg(m_currentNick));
-    } else {
-        //    link.startsWith("##")
+        emit setStatusBarTempText(i18n("Open a query with %1").arg(m_currentNick));
+    }
+    else
+    {
+        // link.startsWith("##")
         m_currentChannel = link.mid(1);
 
         QString prettyId = m_currentChannel;
@@ -246,23 +261,12 @@ void IRCView::highlightedSlot(const QString& link)
 
         m_channelPopup->changeTitle(m_channelPopupId,prettyId);
         m_isOnChannel = true;
-        emit actionStatusText( i18n("Join the channel %1").arg(m_currentChannel));
+        emit setStatusBarTempText(i18n("Join the channel %1").arg(m_currentChannel));
     }
 }
 
-void IRCView::urlClickSlot(const QString &url)
+void IRCView::openLink(const QString& url, bool newTab)
 {
-    urlClickSlot(url,false);
-}
-
-void IRCView::urlClickSlot(const QString& _url, bool newTab)
-{
-    // QTextBrowser bug: a link may be screwed up by other links in the same view, so we
-    // ignore the URL given by highlighted() signal and take our previously remembered
-    // hover URL, which is correct, curiously. -- Eisfuchs (idea by Sho_)
-
-    QString url=_url;
-
     if (!url.isEmpty() && !url.startsWith("#"))
     {
         // Always use KDE default mailer.
@@ -280,7 +284,7 @@ void IRCView::urlClickSlot(const QString& _url, bool newTab)
                     ref.call("newTab", url);
                 }
             } else
-            new KRun(KURL(url));
+                new KRun(KURL::fromPathOrURL(url));
         }
         else
         {
@@ -296,16 +300,20 @@ void IRCView::urlClickSlot(const QString& _url, bool newTab)
             proc->start(KProcess::DontCare);
             delete proc;
         }
-    } else if(url.startsWith("##"))               // Channel
+    }
+    //FIXME: Don't do channel links in DCC Chats to begin with since they don't have a server.
+    else if (url.startsWith("##") && m_server && m_server->isConnected())
     {
         QString channel(url);
         channel.replace("##", "#");
         m_server->sendJoinCommand(channel);
-    } else if(url.startsWith("#"))                // Nick
+    }
+    //FIXME: Don't do user links in DCC Chats to begin with since they don't have a server.
+    else if (url.startsWith("#") && m_server && m_server->isConnected()) 
     {
-        QString recepient(url);
-        recepient.remove("#");
-        NickInfoPtr nickInfo = m_server->obtainNickInfo(recepient);
+        QString recipient(url);
+        recipient.remove("#");
+        NickInfoPtr nickInfo = m_server->obtainNickInfo(recipient);
         m_server->addQuery(nickInfo, true /*we initiated*/);
     }
 }
@@ -329,11 +337,10 @@ bool doHighlight, bool parseURL, bool self)
     KonversationApplication* konvApp = static_cast<KonversationApplication*>(kapp);
 
     // TODO: Use QStyleSheet::escape() here
-    filteredLine.replace("&","&amp;");
     // Replace all < with &lt;
-    filteredLine.replace("<","&lt;");
+    filteredLine.replace("<","\x0blt;");
     // Replace all > with &gt;
-    filteredLine.replace(">","&gt;");
+    filteredLine.replace(">", "\x0bgt;");
 
     #if 0
     if(!Preferences::disableExpansion())
@@ -397,9 +404,6 @@ bool doHighlight, bool parseURL, bool self)
         filteredLine.replace(QRegExp("\003([^0-9]|$)"),"\0031,0\\1");
     }
 
-    // Hack to allow for whois info hostmask info to not be parsed as email
-    filteredLine.replace("&amp;#64;","&#64;");
-
     if(filteredLine.find("\x07") != -1)
     {
         if(Preferences::beep())
@@ -464,16 +468,19 @@ bool doHighlight, bool parseURL, bool self)
     // Highlight
     QString ownNick;
 
-    if(m_server)
+    if (m_server)
     {
         ownNick = m_server->getNickname();
     }
+    else if (m_chatWin->getType() == ChatWindow::DccChat)
+    {
+        ownNick = static_cast<DccChat*>(m_chatWin)->getMyNick();
+    }
 
-    if(doHighlight && m_server && (whoSent != ownNick) && !self)
+    if(doHighlight && (whoSent != ownNick) && !self)
     {
         QString highlightColor;
 
-        // FIXME: We got to get rid of m_server dependance here
         if(Preferences::highlightNick() &&
             filteredLine.lower().find(QRegExp("(^|[^\\d\\w])" +
             QRegExp::escape(ownNick.lower()) +
@@ -573,45 +580,81 @@ bool doHighlight, bool parseURL, bool self)
     return filteredLine;
 }
 
-void IRCView::append(const QString& nick,const QString& message)
+QString IRCView::createNickLine(const QString& nick, bool encapsulateNick)
 {
-    QString channelColor = Preferences::color(Preferences::ChannelMessage).name();
-    QString line;
     QString nickLine = "%2";
-    QString scolor;
-    m_tabNotification = Konversation::tnfNormal;
-
-    if(nick != m_server->getNickname())
-        KonversationApplication::instance()->increaseKarma(nick,1);
 
     if(Preferences::useClickableNicks())
         nickLine = "<a href=\"#" + nick + "\">%2</a>";
 
-    if(Preferences::useColoredNicks())
+    if(encapsulateNick)
+        nickLine = "&lt;" + nickLine + "&gt;";
+
+    if(Preferences::useColoredNicks() && m_server)
     {
-        if(nick != m_server->getNickname())
-            scolor = Preferences::nickColor(m_server->obtainNickInfo(nick)->getNickColor()).name();
+        QString nickColor;
+
+        if (nick != m_server->getNickname())
+            nickColor = Preferences::nickColor(m_server->obtainNickInfo(nick)->getNickColor()).name();
         else
-            scolor =  Preferences::nickColor(8).name();
+            nickColor =  Preferences::nickColor(8).name();
 
-        if(scolor == "#000000")
+        if(nickColor == "#000000")
         {
-            scolor = "#000001";                    // HACK Working around QTextBrowser's auto link coloring
+            nickColor = "#000001";                    // HACK Working around QTextBrowser's auto link coloring
         }
 
-        nickLine = "<font color=\"" + scolor + "\">&lt;"+nickLine+"&gt;</font>";
+        nickLine = "<font color=\"" + nickColor + "\">"+nickLine+"</font>";
     }
-    else
+    //FIXME: Another last-minute hack to get DCC Chat colored nicknames
+    // working. We can't use NickInfo::getNickColor() because we don't
+    // have a server.
+    else if (Preferences::useColoredNicks() && m_chatWin->getType() == ChatWindow::DccChat)
     {
-        nickLine = "&lt;"+nickLine+"&gt;";
-        if(channelColor  == "#000000")
+        QString ownNick = static_cast<DccChat*>(m_chatWin)->getMyNick();
+        QString nickColor;
+
+        if (nick != ownNick)
         {
-            channelColor = "#000001";              // HACK Working around QTextBrowser's auto link coloring
+            int nickvalue = 0;
+
+            for (uint index = 0; index < nick.length(); index++)
+            {
+                nickvalue += nick[index].unicode();
+            }
+
+            nickColor = Preferences::nickColor((nickvalue % 8)).name();
         }
+        else
+            nickColor =  Preferences::nickColor(8).name();
+
+        if(nickColor == "#000000")
+        {
+            nickColor = "#000001";                    // HACK Working around QTextBrowser's auto link coloring
+        }
+
+        nickLine = "<font color=\"" + nickColor + "\">"+nickLine+"</font>";
     }
 
     if(Preferences::useBoldNicks())
-        nickLine = "<b>"+nickLine+"</b>";
+        nickLine = "<b>" + nickLine + "</b>";
+
+    return nickLine;
+}
+
+void IRCView::append(const QString& nick,const QString& message)
+{
+    QString channelColor = Preferences::color(Preferences::ChannelMessage).name();
+
+    if(channelColor  == "#000000")
+    {
+        channelColor = "#000001";              // HACK Working around QTextBrowser's auto link coloring
+    }
+
+    QString line;
+    m_tabNotification = Konversation::tnfNormal;
+
+    QString nickLine = createNickLine(nick);
 
     if(basicDirection(message) == QChar::DirR)
     {
@@ -628,9 +671,24 @@ void IRCView::append(const QString& nick,const QString& message)
 
     emit textToLog(QString("<%1>\t%2").arg(nick).arg(message));
 
+    m_lastInsertionWasLine = false;
+
     doAppend(line);
 }
 
+void IRCView::appendLine()
+{
+    QColor channelColor=Preferences::color(Preferences::ChannelMessage);
+    m_tabNotification = Konversation::tnfNone;
+
+    QString line = "<p><font color=\"" + channelColor.name() + "\"><br><hr color=\""+Preferences::color(Preferences::CommandMessage).name()+"\" noshade></font></p>\n";
+    if (!m_lastInsertionWasLine)
+    {
+        doAppend(line);
+        m_lastInsertionWasLine = true;
+    }
+}
+
 void IRCView::appendRaw(const QString& message, bool suppressTimestamps, bool self)
 {
     QColor channelColor=Preferences::color(Preferences::ChannelMessage);
@@ -646,47 +704,24 @@ void IRCView::appendRaw(const QString& message, bool suppressTimestamps, bool se
         line = QString("<p>" + timeStamp() + " <font color=\"" + channelColor.name() + "\">" + message + "</font></p>\n");
     }
 
+    m_lastInsertionWasLine = false;
+
     doAppend(line, true, self);
 }
 
 void IRCView::appendQuery(const QString& nick,const QString& message)
 {
     QString queryColor=Preferences::color(Preferences::QueryMessage).name();
+
+    if(queryColor  == "#000000")
+    {
+        queryColor = "#000001";                // HACK Working around QTextBrowser's auto link coloring
+    }
+
     QString line;
-    QString nickLine = "%2";
-    QString color;
     m_tabNotification = Konversation::tnfNormal;
 
-    if(nick != m_server->getNickname())
-        KonversationApplication::instance()->increaseKarma(nick,2);
-
-    if(Preferences::useClickableNicks())
-        nickLine = "<a href=\"#" + nick + "\">%2</a>";
-
-    if(Preferences::useColoredNicks())
-    {
-        if(nick != m_server->getNickname())
-            color = Preferences::nickColor(m_server->obtainNickInfo(nick)->getNickColor()).name();
-        else
-            color = Preferences::nickColor(8).name();
-
-        if(color == "#000000")
-        {
-            color = "#000001";                    // HACK Working around QTextBrowser's auto link coloring
-        }
-
-        nickLine = "<font color=\"" + color + "\">&lt;"+nickLine+"&gt;</font>";
-    }
-    else
-    {
-        nickLine = "&lt;"+nickLine+"&gt;";
-        if(queryColor  == "#000000")
-        {
-            queryColor = "#000001";                // HACK Working around QTextBrowser's auto link coloring
-        }
-    }
-    if(Preferences::useBoldNicks())
-        nickLine = "<b>"+nickLine+"</b>";
+    QString nickLine = createNickLine(nick);
 
     if(basicDirection(message) == QChar::DirR)
     {
@@ -703,47 +738,24 @@ void IRCView::appendQuery(const QString& nick,const QString& message)
 
     emit textToLog(QString("<%1>\t%2").arg(nick).arg(message));
 
+    m_lastInsertionWasLine = false;
+
     doAppend(line);
 }
 
 void IRCView::appendAction(const QString& nick,const QString& message)
 {
     QString actionColor=Preferences::color(Preferences::ActionMessage).name();
+
+    if(actionColor  == "#000000")
+    {
+        actionColor = "#000001";               // HACK Working around QTextBrowser's auto link coloring
+    }
+
     QString line;
-    QString nickLine = "%2";
-    QString color;
     m_tabNotification = Konversation::tnfNormal;
 
-    if(nick != m_server->getNickname())
-        KonversationApplication::instance()->increaseKarma(nick,1);
-
-    if(Preferences::useClickableNicks())
-        nickLine = "<a href=\"#" + nick + "\">%2</a>";
-
-    if(Preferences::useColoredNicks())
-    {
-        if(nick != m_server->getNickname())
-            color =Preferences::nickColor( m_server->obtainNickInfo(nick)->getNickColor()).name();
-        else
-            color = Preferences::nickColor(8).name();
-
-        if(color == "#000000")
-        {
-            color = "#000001";                    // HACK Working around QTextBrowser's auto link coloring
-        }
-
-        nickLine = "<font color=\"" + color + "\">"+nickLine+"</font>";
-    }
-    else
-    {
-        if(actionColor  == "#000000")
-        {
-            actionColor = "#000001";               // HACK Working around QTextBrowser's auto link coloring
-        }
-    }
-    if(Preferences::useBoldNicks())
-        nickLine = "<b>"+nickLine+"</b>";
-
+    QString nickLine = createNickLine(nick, false);
 
     if(basicDirection(message) == QChar::DirR)
     {
@@ -760,10 +772,12 @@ void IRCView::appendAction(const QString& nick,const QString& message)
 
     emit textToLog(QString("\t * %1 %2").arg(nick).arg(message));
 
+    m_lastInsertionWasLine = false;
+
     doAppend(line);
 }
 
-void IRCView::appendServerMessage(const QString& type, const QString& message)
+void IRCView::appendServerMessage(const QString& type, const QString& message, bool parseURL)
 {
     QString serverColor = Preferences::color(Preferences::ServerMessage).name();
     m_tabNotification = Konversation::tnfControl;
@@ -772,7 +786,7 @@ void IRCView::appendServerMessage(const QString& type, const QString& message)
     QString fixed;
     if(Preferences::fixedMOTD() && !m_fontDataBase.isFixedPitch(font().family()))
     {
-        if(type == "MOTD")
+        if(type == i18n("MOTD"))
             fixed=" face=\"" + KGlobalSettings::fixedFont().family() + "\"";
     }
 
@@ -789,13 +803,15 @@ void IRCView::appendServerMessage(const QString& type, const QString& message)
         line = "<p><font color=\"" + serverColor + "\"" + fixed + ">%1 <b>[</b>%2<b>]</b> %3</font></p>\n";
     }
 
-    if(type != "Notify")
-        line = line.arg(timeStamp(), type, filter(message,serverColor));
+    if(type != i18n("Notify"))
+        line = line.arg(timeStamp(), type, filter(message, serverColor, 0 , true, parseURL));
     else
         line = "<font color=\"" + serverColor + "\">"+line.arg(timeStamp(), type, message)+"</font>";
 
     emit textToLog(QString("%1\t%2").arg(type).arg(message));
 
+    m_lastInsertionWasLine = false;
+
     doAppend(line);
 }
 
@@ -833,6 +849,8 @@ void IRCView::appendCommandMessage(const QString& type,const QString& message, b
 
     emit textToLog(QString("%1\t%2").arg(type).arg(message));
 
+    m_lastInsertionWasLine = false;
+
     doAppend(line, important, self);
 }
 
@@ -849,7 +867,7 @@ void IRCView::appendBacklogMessage(const QString& firstColumn,const QString& raw
 
     if(!nick.isEmpty() && !nick.startsWith("<") && !nick.startsWith("*"))
     {
-        nick = "|" + nick + "|";
+        nick = '|' + nick + '|';
     }
 
     // Nicks are in "<nick>" format so replace the "<>"
@@ -867,7 +885,9 @@ void IRCView::appendBacklogMessage(const QString& firstColumn,const QString& raw
         line = "<p><font color=\"" + backlogColor + "\">%1 %2 %3</font></p>\n";
     }
 
-    line = line.arg(time, nick, filter(message, backlogColor, NULL, false));
+    line = line.arg(time, nick, filter(message, backlogColor, NULL, false, false));
+
+    m_lastInsertionWasLine = false;
 
     doAppend(line);
 }
@@ -945,7 +965,9 @@ void IRCView::doAppend(const QString& newLine, bool important, bool self)
         }
     }
 
-    if(!m_autoTextToSend.isEmpty())
+    //FIXME: Disable auto-text for DCC Chats since we don't have a server
+    // to parse wildcards.
+    if (!m_autoTextToSend.isEmpty() && m_server)
     {
         // replace placeholders in autoText
         QString sendText = m_server->parseWildcards(m_autoTextToSend,m_server->getNickname(),
@@ -955,23 +977,28 @@ void IRCView::doAppend(const QString& newLine, bool important, bool self)
         // send signal only now
         emit autoText(sendText);
     }
-    if (!m_lastStatusText.isEmpty()) emit clearStatusText();
+    else
+    {
+        m_autoTextToSend = QString::null;
+    }
+
+    if (!m_lastStatusText.isEmpty()) emit clearStatusBarTempText();
 }
 
 // remember if scrollbar was positioned at the end of the text or not
 void IRCView::hideEvent(QHideEvent* /* event */) {
-m_resetScrollbar = ((contentsHeight()-visibleHeight()) == contentsY());
+    m_resetScrollbar = ((contentsHeight()-visibleHeight()) == contentsY());
 }
 
 // Workaround to scroll to the end of the TextView when it's shown
-void IRCView::showEvent(QShowEvent* /* event */) {
-// did the user scroll the view to the end of the text before hiding?
-if(m_resetScrollbar)
-{
-    moveCursor(MoveEnd,false);
-    ensureVisible(0,contentsHeight());
-}
-
+void IRCView::showEvent(QShowEvent* event) {
+    Q_UNUSED(event);
+    // did the user scroll the view to the end of the text before hiding?
+    if(m_resetScrollbar)
+    {
+        moveCursor(MoveEnd,false);
+        ensureVisible(0,contentsHeight());
+    }
 }
 
 void IRCView::contentsMouseReleaseEvent(QMouseEvent *ev)
@@ -980,7 +1007,7 @@ void IRCView::contentsMouseReleaseEvent(QMouseEvent *ev)
     {
         if(m_copyUrlMenu)
         {
-            urlClickSlot(m_urlToCopy,true);
+            openLink(m_urlToCopy,true);
             return;
         }
         else
@@ -994,12 +1021,10 @@ void IRCView::contentsMouseReleaseEvent(QMouseEvent *ev)
     {
         if (m_mousePressed)
         {
-            urlClickSlot(m_urlToDrag);
+            openLink(m_highlightedURL);
             m_mousePressed = false;
             return;
         }
-
-        m_mousePressed = false;
     }
 
     KTextBrowser::contentsMouseReleaseEvent(ev);
@@ -1009,10 +1034,7 @@ void IRCView::contentsMousePressEvent(QMouseEvent* ev)
 {
     if (ev->button() == QMouseEvent::LeftButton)
     {
-        // QTextBrowser bug: a link may be screwed up by other links in the same view, so we
-        // ignore the URL given by highlighted() signal and take our previously remembered
-        // hover URL, which is correct, curiously. -- Eisfuchs (idea by Sho_)
-        m_urlToDrag = m_highlightedURL; //anchorAt(viewportToContents(ev->pos()));
+        m_urlToDrag = m_highlightedURL;
 
         if (!m_urlToDrag.isNull())
         {
@@ -1031,9 +1053,9 @@ void IRCView::contentsMouseMoveEvent(QMouseEvent* ev)
     {
         m_mousePressed = false;
         removeSelection();
-        KURL ux(m_urlToDrag);
+        KURL ux = KURL::fromPathOrURL(m_urlToDrag);
 
-        if (m_urlToDrag.startsWith("##"))
+        if (m_server && m_urlToDrag.startsWith("##"))
         {
             ux = QString("irc://%1:%2/%3").arg(m_server->getServerName()).arg(m_server->getPort()).arg(m_urlToDrag.mid(2));
         }
@@ -1053,32 +1075,57 @@ void IRCView::contentsMouseMoveEvent(QMouseEvent* ev)
 void IRCView::contentsContextMenuEvent(QContextMenuEvent* ev)
 {
     bool block = contextMenu(ev);
+    // Hack to counter the fact that we're given an decoded url
+    m_highlightedURL = KURL::fromPathOrURL(anchorAt(viewportToContents(mapFromGlobal(QCursor::pos())))).url();
 
-    if(!block)
-    {
+    if (m_highlightedURL.isEmpty())
+        viewport()->setCursor(Qt::ArrowCursor);
+
+    if (!block)
         KTextBrowser::contentsContextMenuEvent(ev);
-    }
 }
 
 bool IRCView::contextMenu(QContextMenuEvent* ce)
 {
-    if(m_isOnNick)
+    if (m_server && m_isOnNick && m_nickPopup->isEnabled())
     {
+        updateNickMenuEntries(m_nickPopup, getContextNick());
+
         m_nickPopup->exec(ce->globalPos());
         m_isOnNick = false;
     }
-    else if(m_isOnChannel)
+    else if (m_server && m_isOnChannel && m_channelPopup->isEnabled())
     {
         m_channelPopup->exec(ce->globalPos());
         m_isOnChannel = false;
     }
     else
     {
+        KActionCollection* actionCollection = KonversationApplication::instance()->getMainWindow()->actionCollection();
+        KToggleAction* toggleMenuBarAction = static_cast<KToggleAction*>(actionCollection->action("options_show_menubar"));
+
+        if (toggleMenuBarAction && !toggleMenuBarAction->isChecked())
+        {
+            toggleMenuBarAction->plug(m_popup, 0);
+            m_popup->setItemVisible(toggleMenuBarSeparator, true);
+        }
+
         m_popup->setItemEnabled(Copy,(hasSelectedText()));
 
+        KAction* channelSettingsAction = 0;
+
+        if (m_chatWin->getType() == ChatWindow::Channel)
+        {
+            channelSettingsAction = KonversationApplication::instance()->getMainWindow()->actionCollection()->action("channel_settings");
+            if (channelSettingsAction) channelSettingsAction->plug(m_popup);
+        }
+
+        if (m_chatWin->getType() == ChatWindow::Query)
+            updateNickMenuEntries(m_popup, m_chatWin->getName());
+
         int r = m_popup->exec(ce->globalPos());
 
-        switch(r)
+        switch (r)
         {
             case -1:
                 // dummy. -1 means, no entry selected. we don't want -1to go in default, so
@@ -1114,6 +1161,14 @@ bool IRCView::contextMenu(QContextMenuEvent* ce)
             default:
                 emit extendedPopup(r);
         }
+
+        if (toggleMenuBarAction)
+        {
+            toggleMenuBarAction->unplug(m_popup);
+            m_popup->setItemVisible(toggleMenuBarSeparator, false);
+        }
+
+        if (channelSettingsAction) channelSettingsAction->unplug(m_popup);
     }
     return true;
 }
@@ -1124,22 +1179,19 @@ void IRCView::setupNickPopupMenu()
     m_modes = new KPopupMenu(this,"nicklist_modes_context_submenu");
     m_kickban = new KPopupMenu(this,"nicklist_kick_ban_context_submenu");
     m_nickPopupId= m_nickPopup->insertTitle(m_currentNick);
+
+    m_nickPopup->insertItem(i18n("&Whois"),Konversation::Whois);
+    m_nickPopup->insertItem(i18n("&Version"),Konversation::Version);
+    m_nickPopup->insertItem(i18n("&Ping"),Konversation::Ping);
+
+    m_nickPopup->insertSeparator();
+
     m_modes->insertItem(i18n("Give Op"),Konversation::GiveOp);
     m_modes->insertItem(i18n("Take Op"),Konversation::TakeOp);
     m_modes->insertItem(i18n("Give Voice"),Konversation::GiveVoice);
     m_modes->insertItem(i18n("Take Voice"),Konversation::TakeVoice);
     m_nickPopup->insertItem(i18n("Modes"),m_modes,Konversation::ModesSub);
-    m_nickPopup->insertSeparator();
-    m_nickPopup->insertItem(i18n("&Whois"),Konversation::Whois);
-    m_nickPopup->insertItem(i18n("&Version"),Konversation::Version);
-    m_nickPopup->insertItem(i18n("&Ping"),Konversation::Ping);
-    m_nickPopup->insertSeparator();
-    m_nickPopup->insertItem(i18n("Open Query"),Konversation::OpenQuery);
-    if (kapp->authorize("allow_downloading"))
-    {
-        m_nickPopup->insertItem(SmallIcon("2rightarrow"),i18n("Send &File..."),Konversation::DccSend);
-    }
-    m_nickPopup->insertSeparator();
+
     m_kickban->insertItem(i18n("Kick"),Konversation::Kick);
     m_kickban->insertItem(i18n("Kickban"),Konversation::KickBan);
     m_kickban->insertItem(i18n("Ban Nickname"),Konversation::BanNick);
@@ -1154,27 +1206,78 @@ void IRCView::setupNickPopupMenu()
     m_kickban->insertItem(i18n("Kickban *!user@*.host"),Konversation::KickBanUserHost);
     m_kickban->insertItem(i18n("Kickban *!user@domain"),Konversation::KickBanUserDomain);
     m_nickPopup->insertItem(i18n("Kick / Ban"),m_kickban,Konversation::KickBanSub);
-    m_nickPopup->insertItem(i18n("Ignore"),Konversation::IgnoreNick);
+
+    m_nickPopup->insertItem(i18n("Ignore"), Konversation::IgnoreNick);
+    m_nickPopup->insertItem(i18n("Unignore"), Konversation::UnignoreNick);
+    m_nickPopup->setItemVisible(Konversation::IgnoreNick, false);
+    m_nickPopup->setItemVisible(Konversation::UnignoreNick, false);
+
+    m_nickPopup->insertSeparator();
+
+    m_nickPopup->insertItem(i18n("Open Query"),Konversation::OpenQuery);
+    if (kapp->authorize("allow_downloading"))
+    {
+        m_nickPopup->insertItem(SmallIcon("2rightarrow"),i18n("Send &File..."),Konversation::DccSend);
+    }
+    m_nickPopup->insertSeparator();
+
+    m_nickPopup->insertItem(i18n("Add to Watched Nicks"), Konversation::AddNotify);
 
     connect(m_nickPopup, SIGNAL(activated(int)), this, SIGNAL(popupCommand(int)));
     connect(m_modes, SIGNAL(activated(int)), this, SIGNAL(popupCommand(int)));
     connect(m_kickban, SIGNAL(activated(int)), this, SIGNAL(popupCommand(int)));
 }
 
+void IRCView::updateNickMenuEntries(QPopupMenu* popup, const QString& nickname)
+{
+    if (popup)
+    {
+        if (Preferences::isIgnored(nickname))
+        {
+            popup->setItemVisible(Konversation::UnignoreNick, true);
+            popup->setItemVisible(Konversation::IgnoreNick, false);
+        }
+        else
+        {
+            popup->setItemVisible(Konversation::IgnoreNick, true);
+            popup->setItemVisible(Konversation::UnignoreNick, false);
+        }
+
+        if (!m_server)
+            popup->setItemEnabled(Konversation::AddNotify, false);
+        else if (!m_server->isConnected())
+            popup->setItemEnabled(Konversation::AddNotify, false);
+        else if (!Preferences::hasNotifyList(m_server->serverGroupSettings()->id()))
+            popup->setItemEnabled(Konversation::AddNotify, false);
+        else if (Preferences::isNotify(m_server->serverGroupSettings()->id(), nickname))
+            popup->setItemEnabled(Konversation::AddNotify, false);
+        else
+            popup->setItemEnabled(Konversation::AddNotify, true);
+    }
+}
+
 void IRCView::setupQueryPopupMenu()
 {
     m_nickPopup = new KPopupMenu(this,"query_context_menu");
-    m_nickPopupId= m_nickPopup->insertTitle(m_currentNick);
-    m_nickPopup->insertItem(i18n("Whois"),Konversation::Whois);
-    m_nickPopup->insertItem(i18n("Version"),Konversation::Version);
-    m_nickPopup->insertItem(i18n("Ping"),Konversation::Ping);
+    m_nickPopupId = m_nickPopup->insertTitle(m_currentNick);
+    m_nickPopup->insertItem(i18n("&Whois"),Konversation::Whois);
+    m_nickPopup->insertItem(i18n("&Version"),Konversation::Version);
+    m_nickPopup->insertItem(i18n("&Ping"),Konversation::Ping);
     m_nickPopup->insertSeparator();
+
+    m_nickPopup->insertItem(i18n("Ignore"), Konversation::IgnoreNick);
+    m_nickPopup->insertItem(i18n("Unignore"), Konversation::UnignoreNick);
+    m_nickPopup->setItemVisible(Konversation::IgnoreNick, false);
+    m_nickPopup->setItemVisible(Konversation::UnignoreNick, false);
+    m_nickPopup->insertSeparator();
+
     if (kapp->authorize("allow_downloading"))
     {
         m_nickPopup->insertItem(SmallIcon("2rightarrow"),i18n("Send &File..."),Konversation::DccSend);
+        m_nickPopup->insertSeparator();
     }
-    m_nickPopup->insertSeparator();
-    m_nickPopup->insertItem(i18n("Ignore"),Konversation::IgnoreNick);
+
+    m_nickPopup->insertItem(i18n("Add to Watched Nicks"), Konversation::AddNotify);
 
     connect(m_nickPopup, SIGNAL(activated(int)), this, SIGNAL(popupCommand(int)));
 }
@@ -1190,34 +1293,24 @@ void IRCView::setupChannelPopupMenu()
     connect(m_channelPopup, SIGNAL(activated(int)), this, SIGNAL(popupCommand(int)));
 }
 
+void IRCView::setNickAndChannelContextMenusEnabled(bool enable)
+{
+    if (m_nickPopup) m_nickPopup->setEnabled(enable);
+    if (m_channelPopup) m_channelPopup->setEnabled(enable);
+}
+
 void IRCView::search()
 {
-    /*    m_caseSensitive = false;
-        m_wholeWords = false;
-        m_forward = false;
-        m_fromCursor = false;
-
-        m_pattern = SearchDialog::search(this, &m_caseSensitive, &m_wholeWords, &m_forward, &m_fromCursor);
-
-        if(!m_fromCursor) {
-            if(m_forward) {
-                m_findParagraph = 1;
-                m_findIndex = 1;
-    } else {
-    m_findParagraph = paragraphs();
-    m_findIndex = paragraphLength(paragraphs());
-    }
-    }
-
-    searchAgain(); */
-
     emit doSearch();
 }
 
 void IRCView::searchAgain()
 {
-
-    if(!m_pattern.isEmpty())
+    if (m_pattern.isEmpty())
+    {
+        emit doSearch();
+    }
+    else
     {
         // next search must begin one index before / after the last search
         // depending on the search direction.
@@ -1280,14 +1373,15 @@ bool wholeWords, bool forward, bool fromCursor)
     return searchNext();
 }
 
-bool IRCView::searchNext()
+bool IRCView::searchNext(bool reversed)
 {
     if (m_pattern.isEmpty())
         return true;
 
+    bool fwd = (reversed ? !m_forward : m_forward);
     // next search must begin one index before / after the last search
     // depending on the search direction.
-    if (m_forward)
+    if (fwd)
     {
         ++m_findIndex;
         if(m_findIndex == paragraphLength(m_findParagraph))
@@ -1309,7 +1403,7 @@ bool IRCView::searchNext()
         }
     }
 
-    return find(m_pattern, m_caseSensitive, m_wholeWords, m_forward,
+    return find(m_pattern, m_caseSensitive, m_wholeWords, fwd,
         &m_findParagraph, &m_findIndex);
 }
 
@@ -1398,9 +1492,9 @@ void IRCView::setChatWin(ChatWindow* chatWin)
     m_chatWin = chatWin;
 
     if(m_chatWin->getType()==ChatWindow::Channel)
-      setupNickPopupMenu();   // for channels
+        setupNickPopupMenu();
     else
-      setupQueryPopupMenu();  // for queries
+        setupQueryPopupMenu();
 
     setupChannelPopupMenu();
 }
