@@ -6,10 +6,11 @@
 */
 
 /*
-  The main application
-  begin:     Mon Jan 28 2002
-  copyright: (C) 2002 by Dario Abatianni
-  email:     eisfuchs@tigress.com
+  Copyright (C) 2002 Dario Abatianni <eisfuchs@tigress.com>
+  Copyright (C) 2005 Ismail Donmez <ismail@kde.org>
+  Copyright (C) 2005 Peter Simonsson <psn@linux.se>
+  Copyright (C) 2005 John Tapsell <johnflux@gmail.com>
+  Copyright (C) 2005 Eike Hein <sho@eikehein.com>
 */
 
 #include <qtextcodec.h>
@@ -27,6 +28,7 @@
 
 #include "konversationapplication.h"
 #include "konversationmainwindow.h"
+#include "viewcontainer.h"
 #include "highlight.h"
 #include "server.h"
 #include "konversationsound.h"
@@ -45,18 +47,12 @@ KonversationApplication::KonversationApplication()
 {
     mainWindow = 0;
     quickConnectDialog = 0;
-    colorOffSet = 0;
-    m_demoteInProgress = false;
     m_connectDelayed=false;
-
-    demoteTimer = new QTimer(this);
-    connect(demoteTimer,SIGNAL(timeout()),this,SLOT(autoDemoteAllNicks()) );
-    demoteTimer->start(5*60*1000,false);
-
 }
 
 KonversationApplication::~KonversationApplication()
 {
+    Preferences::writeConfig();
     saveOptions(false);
 
     delete m_images;
@@ -84,7 +80,6 @@ int KonversationApplication::newInstance()
     {
         // make sure all vars are initialized properly
         quickConnectDialog = 0;
-        colorOffSet = 0;
 
         // Sound object used to play sound...
         m_sound = new Konversation::Sound(this);
@@ -97,6 +92,7 @@ int KonversationApplication::newInstance()
 
         // Images object providing LEDs, NickIcons
         m_images = new Images();
+        connect(this, SIGNAL(iconChanged(int)), m_images, SLOT(updateIcons()));
 
         // Auto-alias scripts.  This adds any missing aliases
         QStringList aliasList(Preferences::aliasList());
@@ -124,29 +120,21 @@ int KonversationApplication::newInstance()
         connect(Preferences::self(), SIGNAL(updateTrayIcon()), mainWindow, SLOT(updateTrayIcon()) );
         connect(this, SIGNAL(prefsChanged()), mainWindow, SIGNAL(prefsChanged()) );
         // take care of user style changes, setting back colors and stuff
-        connect(KApplication::kApplication(), SIGNAL(appearanceChanged()), mainWindow, SLOT(updateAppearance()));
-        connect(KApplication::kApplication(), SIGNAL(appearanceChanged()), mainWindow, SLOT(updateTabs()));
 
         // apply GUI settings
-        mainWindow->updateAppearance();
+        emit appearanceChanged();
         mainWindow->show();
 
-        if(Preferences::showServerList())
-        {
-            mainWindow->openServerList();
-        }
+        if (Preferences::showServerList()) mainWindow->openServerList();
 
         // handle autoconnect on startup
         Konversation::ServerGroupList serverGroups = Preferences::serverGroupList();
 
-        if(!m_connectDelayed)
+        if (!m_connectDelayed)
         {
-            for(Konversation::ServerGroupList::iterator it = serverGroups.begin(); it != serverGroups.end(); ++it)
+            for (Konversation::ServerGroupList::iterator it = serverGroups.begin(); it != serverGroups.end(); ++it)
             {
-                if((*it)->autoConnectEnabled())
-                {
-                    connectToServer((*it)->id());
-                }
+                if ((*it)->autoConnectEnabled()) connectToServer((*it)->id());
             }
         }
         else
@@ -168,9 +156,7 @@ int KonversationApplication::newInstance()
             connect(dcopObject,SIGNAL (dcopInfo(const QString&)),
                 this,SLOT (dcopInfo(const QString&)) );
             connect(dcopObject,SIGNAL (dcopInsertRememberLine()),
-                mainWindow,SLOT (insertRememberLine()));
-            connect(dcopObject,SIGNAL (dcopSetAutoAway()),
-                this,SLOT (setAutoAway()));
+                mainWindow,SIGNAL(insertRememberLine()));
             connect(dcopObject,SIGNAL(dcopConnectToServer(const QString&, int,const QString&, const QString&)),
                 this,SLOT(dcopConnectToServer(const QString&, int,const QString&, const QString&)));
         }
@@ -192,20 +178,6 @@ KonversationMainWindow *KonversationApplication::getMainWindow()
     return mainWindow;
 }
 
-void KonversationApplication::setAutoAway()
-{
-    Server* lookServer=serverList.first();
-    while(lookServer)
-    {
-        if(!lookServer->isAway())
-        {
-            lookServer->setAutoAway();
-        }
-        lookServer=serverList.next();
-    }
-
-}
-
 void KonversationApplication::toggleAway()
 {
     bool anyservers = false;
@@ -228,18 +200,10 @@ void KonversationApplication::toggleAway()
     }
 
     //alreadyaway is true if _any_ servers are away
-    if(alreadyaway)
-    {
-                                                  //toggle as not away
-        sendMultiServerCommand("away", QString::null);
-    }
+    if (alreadyaway)
+        sendMultiServerCommand("back", QString::null);
     else
-    {
-        QString awaymessage ;                     //get default awaymessage
-        if(awaymessage.isEmpty()) awaymessage = "Away at the moment";
-
-        sendMultiServerCommand("away", awaymessage);
-    }
+        sendMultiServerCommand("away", QString::null);
 }
 
 void KonversationApplication::dcopMultiServerRaw(const QString &command)
@@ -273,7 +237,7 @@ void KonversationApplication::dcopSay(const QString& server,const QString& targe
             lookServer->dcopSay(target,command);
             //      break; // leave while loop
             //FIXME:   <muesli> there's a reason for not breaking this loop, here (which would spent only some
-            //                  cpu cycles, anyways): i'm connected to two bouncers at the same time, which are
+            //                  cpu cycles, anyways): I am connected to two bouncers at the same time, which are
             //                  also named the same (same ip, no dns). if a dcopSay gets emerged, it will always
             //                  get the _same_ server name as its parameter (both are named the same). although
             //                  the channel it gets sent to, is on the second server, it will always try to send
@@ -286,7 +250,51 @@ void KonversationApplication::dcopSay(const QString& server,const QString& targe
 
 void KonversationApplication::dcopInfo(const QString& string)
 {
-    mainWindow->appendToFrontmost(i18n("DCOP"), string, 0);
+    mainWindow->getViewContainer()->appendToFrontmost(i18n("DCOP"), string, 0);
+}
+
+bool KonversationApplication::validateIdentity(IdentityPtr identity, bool interactive)
+{
+    QString errors;
+
+    if (identity->getIdent().isEmpty())
+        errors+=i18n("Please fill in your <b>Ident</b>.<br>");
+
+    if (identity->getRealName().isEmpty())
+        errors+=i18n("Please fill in your <b>Real name</b>.<br>");
+
+    if (identity->getNickname(0).isEmpty())
+        errors+=i18n("Please provide at least one <b>Nickname</b>.<br>");
+
+    if (!errors.isEmpty())
+    {
+        if (interactive)
+        {
+            int result = KMessageBox::warningContinueCancel(0,
+                            i18n("<qt>Your identity \"%1\" is not set up correctly:<br>%2</qt>")
+                                .arg(identity->getName()).arg(errors),
+                            i18n("Identity Settings"),
+                            i18n("Edit Identity..."));
+
+            if (result==KMessageBox::Continue)
+            {
+                identity = mainWindow->editIdentity(identity);
+
+                if (identity && validateIdentity(identity,false))
+                    return true;
+                else
+                    return false;
+            }
+            else
+            {
+                return false;
+            }
+        }
+
+        return false;
+    }
+
+    return true;
 }
 
 Server* KonversationApplication::connectToServerGroup(const QString& serverGroup)
@@ -296,56 +304,97 @@ Server* KonversationApplication::connectToServerGroup(const QString& serverGroup
     return connectToServer(serverGroupId);
 }
 
-Server* KonversationApplication::connectToServer (int serverGroupId, Konversation::ServerSettings quickServer)
+Server* KonversationApplication::connectToServer(int serverGroupId, Konversation::ServerSettings quickServer)
 {
     // Check if a server window with same name and port is already open
     Server* lookServer = serverList.first();
+    Server* existingServer = 0;
 
     while (lookServer)
     {
       if (lookServer->serverGroupSettings()->id() == serverGroupId)
-          return lookServer;
+      {
+          existingServer = lookServer;
+          break;
+      }
 
       lookServer = serverList.next();
     }
 
+    // There's already a connection to this network.
+    if (existingServer)
+    {
+        // The connection is active, and the server list dialog requested
+        // to connect to a specific server.
+        if (existingServer->isConnected() && !quickServer.server().isEmpty())
+        {
+            // This server differs from the server we're presently connected to.
+            if (quickServer.server() != existingServer->getServerName())
+            {
+                int result = KMessageBox::warningContinueCancel(
+                mainWindow,
+                i18n("You are already connected to %1. Do you want to disconnect from '%2' and connect to '%3' instead?")
+                    .arg(existingServer->getServerGroup())
+                    .arg(existingServer->getServerName())
+                    .arg(quickServer.server()),
+                i18n("Already connected to %1").arg(existingServer->getServerGroup()),
+                i18n("Disconnect"),
+                "ReconnectDifferentServer");
+
+                // The user has chosen to connect to this different server
+                // instead of the current one.
+                if (result == KMessageBox::Continue)
+                {
+                    existingServer->disconnect();
+                    existingServer->serverGroupSettings()->clearQuickServerList();
+                    existingServer->serverGroupSettings()->setQuickServerList(quickServer);
+                    existingServer->resetCurrentServerIndex();
+                    existingServer->reconnect();
+                    return existingServer;
+                }
+                // The user wants to keep the current connection alive, so
+                // return the existing connection.
+                else
+                    return existingServer;
+            }
+            // We've gotten a request to connect to the same server we're
+            // already connected to, so return the existing connection.
+            else
+                return existingServer;
+        }
+        // We haven't been told to connect to a specific server and already have
+        // a connection to this network, so return it.
+        else if (existingServer->isConnected() && quickServer.server().isEmpty())
+        {
+            return existingServer;
+        }
+        // The connection is inactive.
+        else if (!existingServer->isConnected())
+        {
+            // We've been told to connect to a specific server. Do so.
+            if (!quickServer.server().isEmpty())
+            {
+                existingServer->serverGroupSettings()->clearQuickServerList();
+                existingServer->serverGroupSettings()->setQuickServerList(quickServer);
+                existingServer->resetCurrentServerIndex();
+                existingServer->reconnect();
+                return existingServer;
+            }
+            // No specific server was part of the request, so reconnect
+            // the old one.
+            else
+            {
+                existingServer->reconnect();
+                return existingServer;
+            }
+        }
+    }
+
     Konversation::ServerGroupSettingsPtr serverGroup = Preferences::serverGroupById(serverGroupId);
     IdentityPtr identity = serverGroup->identity();
 
-    if(!identity)
-    {
+    if (!identity || !validateIdentity(identity))
         return 0;
-    }
-
-    // sanity check for identity
-    QString check;
-
-    if(identity->getIdent().isEmpty())
-    {
-        check+=i18n("Please fill in your <b>ident</b>.<br>");
-    }
-    if(identity->getRealName().isEmpty())
-    {
-        check+=i18n("Please fill in your <b>Real name</b>.<br>");
-    }
-    if(identity->getNickname(0).isEmpty())
-    {
-        check+=i18n("Please provide at least one <b>Nickname</b>.<br>");
-    }
-    if(!check.isEmpty())
-    {
-        KMessageBox::information(0,
-            i18n("<qt>Your identity \"%1\" is not set up correctly:<br>%2</qt>")
-            .arg(identity->getName()).arg(check),
-            i18n("Check Identity Settings")
-            );
-
-        mainWindow->openIdentitiesDialog();
-
-        return 0;
-    }
-
-    // identity ok, carry on
 
     emit closeServerList();
 
@@ -362,12 +411,12 @@ Server* KonversationApplication::connectToServer (int serverGroupId, Konversatio
 
     mainWindow->show();
 
-    Server* newServer = new Server(mainWindow, serverGroupId, clearQuickServerList);
+    Server* newServer = new Server(mainWindow->getViewContainer(), serverGroupId, clearQuickServerList);
 
     connect(mainWindow,SIGNAL (startNotifyTimer(int)),newServer,SLOT (startNotifyTimer(int)) );
     connect(mainWindow,SIGNAL (quitServer()),newServer,SLOT (quitServer()) );
     connect(newServer, SIGNAL(connectionChangedState(Server*, Server::State)),
-        mainWindow, SLOT(serverStateChanged(Server*, Server::State)));
+        mainWindow, SIGNAL(serverStateChanged(Server*, Server::State)));
 
     connect(newServer,SIGNAL (nicksNowOnline(Server*,const QStringList&,bool)),mainWindow,SLOT (setOnlineList(Server*,const QStringList&,bool)) );
 
@@ -375,7 +424,7 @@ Server* KonversationApplication::connectToServer (int serverGroupId, Konversatio
 
     connect(newServer, SIGNAL(multiServerCommand(const QString&, const QString&)),
         this, SLOT(sendMultiServerCommand(const QString&, const QString&)));
-    connect(newServer, SIGNAL(awayInsertRememberLine(Server*)), mainWindow, SLOT(insertRememberLine(Server*)));
+    connect(newServer, SIGNAL(awayInsertRememberLine(Server*)), mainWindow, SIGNAL(insertRememberLine(Server*)));
 
     serverList.append(newServer);
 
@@ -386,10 +435,24 @@ void KonversationApplication::quickConnectToServer(const QString& hostName, cons
 {
     //used for the quick connect dialog and /server command
 
-    Server* newServer = new Server(mainWindow, hostName, port, channel, nick, password, useSSL);
+    IdentityPtr identity;
+    Konversation::ServerGroupSettingsPtr serverGroupOfServer;
+
+    // If server is in an existing group, use that group (first group if server is in multiple groups)
+    if (serverGroupOfServer = Preferences::serverGroupByServer(hostName))
+        identity = serverGroupOfServer->identity();
+    else
+        identity = Preferences::identityByName("Default");
+
+    if (!identity || !validateIdentity(identity))
+        return;
+
+    Server* newServer = new Server(mainWindow->getViewContainer(), hostName, port, channel, nick, password, useSSL);
 
     connect(mainWindow,SIGNAL (startNotifyTimer(int)),newServer,SLOT (startNotifyTimer(int)) );
     connect(mainWindow,SIGNAL (quitServer()),newServer,SLOT (quitServer()) );
+    connect(newServer, SIGNAL(connectionChangedState(Server*, Server::State)),
+        mainWindow, SIGNAL(serverStateChanged(Server*, Server::State)));
 
     connect(newServer,SIGNAL (nicksNowOnline(Server*,const QStringList&,bool)),mainWindow,SLOT (setOnlineList(Server*,const QStringList&,bool)) );
 
@@ -397,7 +460,7 @@ void KonversationApplication::quickConnectToServer(const QString& hostName, cons
 
     connect(newServer, SIGNAL(multiServerCommand(const QString&, const QString&)),
         this, SLOT(sendMultiServerCommand(const QString&, const QString&)));
-    connect(newServer, SIGNAL(awayInsertRememberLine(Server*)), mainWindow, SLOT(insertRememberLine(Server*)));
+    connect(newServer, SIGNAL(awayInsertRememberLine(Server*)), mainWindow, SIGNAL(insertRememberLine(Server*)));
 
     serverList.append(newServer);
 }
@@ -422,11 +485,6 @@ void KonversationApplication::removeServer(Server* server)
         kdDebug() << "Could not remove " << server->getServerName() << endl;
 }
 
-void KonversationApplication::quitKonversation()
-{
-    qApp->quit();
-}
-
 void KonversationApplication::readOptions()
 {
     // get standard config file
@@ -437,12 +495,11 @@ void KonversationApplication::readOptions()
     QString sortOrder=config->readEntry("SortOrder");
     QStringList sortOrderList=QStringList::split("",sortOrder);
     sortOrderList.sort();
-    if(sortOrderList.join("")!="-hopqv")
+    if (sortOrderList.join("")!="-hopqv")
     {
-      kdDebug() << "KonversationApplication::readOptions: Broken nick list sort order found, resetting to defaults." << endl;
       sortOrder=Preferences::defaultNicknameSortingOrder();
+      Preferences::setSortOrder(sortOrder);
     }
-    Preferences::setSortOrder(sortOrder);
 
     // Identity list
     QStringList identityList=config->groupList().grep(QRegExp("Identity [0-9]+"));
@@ -591,17 +648,44 @@ void KonversationApplication::readOptions()
     Preferences::setUseNotify(Preferences::useNotify());
 
     // Quick Buttons List
+
+    // if there are button definitions in the config file, remove default buttons
+    if(config->hasGroup("Button List")) Preferences::clearQuickButtonList();
     config->setGroup("Button List");
-    // Read all buttons and overwrite default entries
+    // Read all default buttons
     QStringList buttonList(Preferences::quickButtonList());
-    for(int index=0;index<8;index++)
+    // Read all quick buttons
+    int index=0;
+    while(config->hasKey(QString("Button%1").arg(index)))
     {
-        QString buttonKey(QString("Button%1").arg(index));
-        if(config->hasKey(buttonKey)) buttonList[index]=config->readEntry(buttonKey);
-    }
+      buttonList.append(config->readEntry(QString("Button%1").arg(index++)));
+    } // while
     // Put back the changed button list
     Preferences::setQuickButtonList(buttonList);
 
+    // Autoreplace List
+
+    // if there are autoreplace definitions in the config file, remove default entries
+    if(config->hasGroup("Autoreplace List")) Preferences::clearAutoreplaceList();
+    config->setGroup("Autoreplace List");
+    // Read all default entries
+    QStringList autoreplaceList(Preferences::autoreplaceList());
+    // Read all entries
+    index=0;
+    while(config->hasKey(QString("Autoreplace%1").arg(index)))
+    {
+      // read entry and get length of the string
+      QString entry=config->readEntry(QString("Autoreplace%1").arg(index++));
+      unsigned int length=entry.length()-1;
+      // if there's a "#" in the end, strip it (used to preserve blanks at the end of the replacement text)
+      // there should always be one, but older versions did not do it, so we check first
+      if(entry.at(length)=='#') entry=entry.left(length);
+      // add entry to internal list
+      autoreplaceList.append(entry);
+    } // while
+    // Put back the changed autoreplace list
+    Preferences::setAutoreplaceList(autoreplaceList);
+
     // Highlight List
     if(config->hasKey("Highlight"))               // Stay compatible with versions < 0.14
     {
@@ -611,7 +695,7 @@ void KonversationApplication::readOptions()
         unsigned int hiIndex;
         for(hiIndex=0;hiIndex<hiList.count();hiIndex+=2)
         {
-            Preferences::addHighlight(hiList[hiIndex],false,"#"+hiList[hiIndex+1],QString::null,QString::null);
+            Preferences::addHighlight(hiList[hiIndex],false,'#'+hiList[hiIndex+1],QString::null,QString::null);
         }
 
         config->deleteEntry("Highlight");
@@ -637,7 +721,7 @@ void KonversationApplication::readOptions()
     // Remove all default entries if there is at least one Ignore in the Preferences::file
     if(config->hasKey("Ignore0")) Preferences::clearIgnoreList();
     // Read all ignores
-    int index=0;
+    index=0;
     while(config->hasKey(QString("Ignore%1").arg(index)))
     {
         Preferences::addIgnore(config->readEntry(QString("Ignore%1").arg(index++)));
@@ -810,18 +894,6 @@ void KonversationApplication::saveOptions(bool updateGUI)
     }
 
     config->deleteGroup("Server List");
-/*
-    Should be done in QuickButtonConfigController now ...
-    remove this part as soon as we are certain it works
-
-    config->setGroup("Button List");
-
-    for(index=0;index<8;index++)
-    {
-        QStringList buttonList(Preferences::buttonList());
-        config->writeEntry(QString("Button%1").arg(index),buttonList[index]);
-    }
-*/
 /*
     Should be done in HighlightConfigController now ...
     remove this part as soon as we are certain it works
@@ -871,7 +943,7 @@ void KonversationApplication::saveOptions(bool updateGUI)
         channelEncodingsChannelList.sort();
         for(unsigned int j=0; j<channelEncodingsChannelList.count(); ++j)
             if(!Preferences::channelEncoding(channelEncodingsServerList[i],channelEncodingsChannelList[j]).isEmpty())
-                config->writeEntry(channelEncodingsServerList[i]+" "+channelEncodingsChannelList[j],Preferences::channelEncoding(channelEncodingsServerList[i],channelEncodingsChannelList[j]));
+                config->writeEntry(channelEncodingsServerList[i]+' '+channelEncodingsChannelList[j],Preferences::channelEncoding(channelEncodingsServerList[i],channelEncodingsChannelList[j]));
     }
 
     config->sync();
@@ -909,7 +981,7 @@ void KonversationApplication::storeUrl(const QString& who,const QString& newUrl)
 
     // check that we don't add the same URL twice
     deleteUrl(who,url);
-    urlList.append(who+" "+url);
+    urlList.append(who+' '+url);
     emit catchUrl(who,url);
 }
 
@@ -920,7 +992,7 @@ const QStringList& KonversationApplication::getUrlList()
 
 void KonversationApplication::deleteUrl(const QString& who,const QString& url)
 {
-    urlList.remove(who+" "+url);
+    urlList.remove(who+' '+url);
 }
 
 void KonversationApplication::clearUrlList()
@@ -964,21 +1036,6 @@ bool KonversationApplication::emitDCOPSig(const QString &appId, const QString &o
     }
 }
 
-QPtrList<IRCEvent> KonversationApplication::retrieveHooks (EVENT_TYPE a_type)
-{
-    QPtrList<IRCEvent> ret_value;
-    IRCEvent *e;
-
-    for (e = dcopObject->registered_events.first(); e; e = dcopObject->registered_events.next())
-    {
-        if (e->type == a_type)
-        {
-            ret_value.append(e);
-        }
-    }
-    return ret_value;
-}
-
 void KonversationApplication::sendMultiServerCommand(const QString& command, const QString& parameter)
 {
     for(Server* server = serverList.first(); server; server = serverList.next())
@@ -1011,48 +1068,9 @@ Images* KonversationApplication::images()
 // Returns list of pointers to Servers.
 const QPtrList<Server> KonversationApplication::getServerList() { return serverList; }
 
-// keeps track of the cyclic assigned nickname colors
-uint& KonversationApplication::getColorOffset()
-{
-    return colorOffSet;
-}
-
-uint KonversationApplication::getKarma(const QString& nick) const
-{
-    return karmaMap[nick];
-}
-
-void KonversationApplication::increaseKarma(const QString& nick, uint increase)
-{
-    if(!m_demoteInProgress)
-        karmaMap[nick] += increase;
-
-    //kdDebug() << "New karma for " << nick << " is " << karmaMap[nick] << endl;
-}
-
-void KonversationApplication::decreaseKarma(const QString& nick)
-{
-    if(karmaMap[nick] > 0)
-        karmaMap[nick] -= 1;
-
-    //kdDebug() << "New karma for " << nick << " is " << karmaMap[nick] << endl;
-}
-
-void KonversationApplication::autoDemoteAllNicks()
-{
-    m_demoteInProgress = true;
-
-    for(QMap<QString,uint>::Iterator it = karmaMap.begin(); it != karmaMap.end(); ++it)
-    {
-        decreaseKarma(it.key());
-    }
-
-    m_demoteInProgress = false;
-}
-
 void KonversationApplication::splitNick_Server(QString nick_server, QString &ircnick, QString &serverOrGroup)
 {
-    //kaddresbook uses the utf seperator 0xE120, so treat that as a seperator as well
+    //kaddresbook uses the utf separator 0xE120, so treat that as a separator as well
     nick_server = nick_server.replace(QChar(0xE120), "@");
     ircnick = nick_server.section("@",0,0);
     serverOrGroup = nick_server.section("@",1);
@@ -1075,6 +1093,68 @@ NickInfoPtr KonversationApplication::getNickInfo(const QString &ircnick, const Q
     return 0;
 }
 
+// auto replace on input/output
+QString KonversationApplication::doAutoreplace(const QString& text,bool output)
+{
+    // get autoreplace list
+    QStringList autoreplaceList=Preferences::autoreplaceList();
+    // working copy
+    QString line=text;
+
+    // loop through the list of replacement patterns
+    for(unsigned int index=0;index<autoreplaceList.count();index++)
+    {
+        // get autoreplace definition
+        QString definition=autoreplaceList[index];
+        // split definition in parts
+        QString regex=definition.section(',',0,0);
+        QString direction=definition.section(',',1,1);
+        QString pattern=definition.section(',',2,2);
+        QString replacement=definition.section(',',3);
+
+        QString isDirection=output ? "o" : "i";
+
+        // only replace if this pattern is for the specific direction or both directions
+        if(direction==isDirection || direction=="io")
+        {
+            // regular expression pattern?
+            if(regex=="1")
+            {
+                // create regex from pattern
+                QRegExp needleReg=pattern;
+                // set pattern case insensitive
+                needleReg.setCaseSensitive(false);
+                // find matches
+                if(line.find(needleReg)!=-1)
+                {
+                    // prepare list of captured ( ) groups
+                    QStringList captures;
+                    // remember captured patterns
+                    captures=needleReg.capturedTexts();
+
+                    // replace %0 - %9 in regex groups
+                    for(unsigned int capture=0;capture<captures.count();capture++)
+                    {
+                        replacement.replace(QString("%%1").arg(capture),captures[capture]);
+                    }
+                    replacement.replace(QRegExp("%[0-9]"),QString::null);
+                    // replace input with replacement
+                    line.replace(needleReg,replacement);
+                }
+            }
+            else
+            {
+                QRegExp needleReg("\\b" + QRegExp::escape(pattern) + "\\b");
+                needleReg.setCaseSensitive(false);
+                line.replace(needleReg,replacement);
+            }
+        }
+    }
+
+  return line;
+}
+
 #include "konversationapplication.moc"
 
+// kate: space-indent on; tab-width 4; indent-width 4; mixed-indent off; replace-tabs on;
 // vim: set et sw=4 ts=4 cino=l1,cs,U1:
