@@ -6,10 +6,11 @@
 */
 
 /*
-  The main application
-  begin:     Mon Jan 28 2002
-  copyright: (C) 2002 by Dario Abatianni
-  email:     eisfuchs@tigress.com
+  Copyright (C) 2002 Dario Abatianni <eisfuchs@tigress.com>
+  Copyright (C) 2005 Ismail Donmez <ismail@kde.org>
+  Copyright (C) 2005 Peter Simonsson <psn@linux.se>
+  Copyright (C) 2005 John Tapsell <johnflux@gmail.com>
+  Copyright (C) 2005 Eike Hein <sho@eikehein.com>
 */
 
 #ifndef KONVERSATIONAPPLICATION_H
@@ -18,7 +19,6 @@
 #include <kuniqueapplication.h>
 
 #include "preferences.h"
-#include "ircevent.h"
 #include "osd.h"
 #include "konvdcop.h"
 #include "identity.h"
@@ -41,12 +41,12 @@ namespace Konversation
     enum PopupIDs
     {
         ModesSub,GiveOp,TakeOp,GiveHalfOp,TakeHalfOp,GiveVoice,TakeVoice,
-        KickBanSub,IgnoreNick,
+        KickBanSub,IgnoreNick,UnignoreNick,
         Kick,KickBan,BanNick,BanHost,BanDomain,BanUserHost,BanUserDomain,
         KickBanHost,KickBanDomain,KickBanUserHost,KickBanUserDomain,
         Whois,Version,Ping,OpenQuery,DccSend,Join,Names,Topic,
         CustomID, AddressbookSub, AddressbookChange, AddressbookNew, AddressbookDelete,
-        AddressbookEdit, SendEmail, StartDccChat
+        AddressbookEdit, SendEmail, StartDccChat, AddNotify
     };
 
 }
@@ -55,7 +55,7 @@ class KonversationApplication : public KUniqueApplication
 {
     Q_OBJECT
 
-        public:
+    public:
         /** This function in general shouldn't be called, because in the future there
          *  may be multiple windows.
          *  However, in some situations we have messageboxes that aren't targeted for
@@ -69,9 +69,6 @@ class KonversationApplication : public KUniqueApplication
         void storeUrl(const QString& who,const QString& url);
         const QStringList& getUrlList();
 
-        // DCOP: Returns a list of signals we should emit
-        QPtrList<IRCEvent> retrieveHooks(EVENT_TYPE type);
-
         KonversationApplication();
         ~KonversationApplication();
 
@@ -80,7 +77,7 @@ class KonversationApplication : public KUniqueApplication
         Server* getServerByName(const QString& name);
 
         /** For dcop and addressbook, a user can be specified as user@irc.server.net
-         *  or user@servergroup or using the unicode seperator symbol 0xE120 instead
+         *  or user@servergroup or using the unicode separator symbol 0xE120 instead
          *  of the "@".  This function takes a string like the above examples, and
          *  modifies ircnick and serverOrGroup to contain the split up string.  If
          *  the string doesn't have an @ or 0xE120, ircnick is set to the
@@ -99,6 +96,8 @@ class KonversationApplication : public KUniqueApplication
          */
         NickInfoPtr getNickInfo(const QString &ircnick, const QString &serverOrGroup);
 
+        bool validateIdentity(IdentityPtr identity, bool interactive=true);
+
         OSDWidget* osd;
 
         Konversation::Sound* sound();
@@ -113,13 +112,8 @@ class KonversationApplication : public KUniqueApplication
 
         Konversation::NotificationHandler* notificationHandler() const { return m_notificationHandler; }
 
-        // Nick color stuff
-        uint& getColorOffset();
-
-        // Intelligent nick completion stuff aka A.W.E.S.O.M.E-O 4000 --cartman
-        uint getKarma(const QString& nick) const;
-        void increaseKarma(const QString& nick, uint increase);
-        void decreaseKarma(const QString& nick);  // A decrease is always -1 and its auto
+        // auto replacement for input or output lines
+        QString doAutoreplace(const QString& text,bool output);
 
         int newInstance();
 
@@ -128,10 +122,10 @@ class KonversationApplication : public KUniqueApplication
             const QString& channel="",
             const QString& nick = Preferences::nickname(0),
             const QString& password="",
-            const bool& useSSL=FALSE
+            const bool& useSSL=false
             );
 
-        signals:
+    signals:
         void catchUrl(const QString& who,const QString& url);
         void prefsChanged();
         void closeServerList();
@@ -144,11 +138,10 @@ class KonversationApplication : public KUniqueApplication
             const QString& channel="",
             const QString& nick = Preferences::nickname(0),
             const QString& password="",
-            const bool& useSSL=FALSE
+            const bool& useSSL=false
             );
         void readOptions();
         void saveOptions(bool updateGUI=true);
-        void quitKonversation();
 
         void deleteUrl(const QString& who,const QString& url);
         void clearUrlList();
@@ -166,11 +159,8 @@ class KonversationApplication : public KUniqueApplication
         void dcopRaw(const QString& server, const QString &command);
         void dcopSay(const QString& server,const QString& target,const QString& command);
         void dcopInfo(const QString& string);
-        void setAutoAway();
         void sendMultiServerCommand(const QString& command, const QString& parameter);
 
-        void autoDemoteAllNicks(); // Auto decrease karmas for all nicks, this is a QTimer based operation
-
     private:
         QPtrList<Server> serverList;
         QStringList urlList;
@@ -183,13 +173,8 @@ class KonversationApplication : public KUniqueApplication
 
         Konversation::NotificationHandler* m_notificationHandler;
 
-        uint colorOffSet;
         QStringList colorList;
 
-        bool m_demoteInProgress;
-        QTimer* demoteTimer;
-        QMap<QString,uint> karmaMap;
-
         // For command line arguments
         QString m_hostName;
         QString m_port;
@@ -199,4 +184,8 @@ class KonversationApplication : public KUniqueApplication
         bool m_useSSL;
         bool m_connectDelayed;
 };
+
 #endif
+
+// kate: space-indent on; tab-width 4; indent-width 4; mixed-indent off; replace-tabs on;
+// vim: set et sw=4 ts=4 cino=l1,cs,U1:
