@@ -6,9 +6,8 @@
 */
 
 /*
-  begin:     Mon Jan 28 2002
-  copyright: (C) 2002 by Dario Abatianni
-  email:     eisfuchs@tigress.com
+  Copyright (C) 2002 Dario Abatianni <eisfuchs@tigress.com>
+  Copyright (C) 2005 Eike Hein <sho@eikehein.com>
 */
 
 #ifndef QUERY_H
@@ -29,10 +28,14 @@
 class QLineEdit;
 class QCheckBox;
 class QLabel;
-class QStrList;
+class QSplitter;
 
 class IRCInput;
 
+namespace Konversation {
+  class TopicLabel;
+}
+
 class Query : public ChatWindow
 {
     Q_OBJECT
@@ -60,8 +63,7 @@ class Query : public ChatWindow
         virtual QString getChannelEncodingDefaultDesc();
         virtual void emitUpdateInfo();
 
-        virtual bool areIRCColorsSupported() {return true; }
-        virtual bool isInsertCharacterSupported() { return true; }
+        virtual bool isInsertSupported() { return true; }
 
         /** call this when you see a nick quit from the server.
          *  @param reason The quit reason given by that user.
@@ -77,7 +79,6 @@ class Query : public ChatWindow
         void appendInputText(const QString& s);
         virtual void indicateAway(bool show);
         void updateAppearance();
-        virtual void lostFocus();
 
     protected slots:
         void queryTextEntered();
@@ -88,6 +89,7 @@ class Query : public ChatWindow
         void textPasted(const QString& text);
         void popup(int id);
         void nickInfoChanged();
+        virtual void serverOnline(bool online);
 
     protected:
         void setName(const QString& newName);
@@ -101,11 +103,14 @@ class Query : public ChatWindow
         QString queryName;
         QString buffer;
 
-        QLabel* queryHostmask;
+        QSplitter* m_headerSplitter;
+        Konversation::TopicLabel* queryHostmask;
         QLabel* addresseeimage;
         QLabel* addresseelogoimage;
         QLabel* awayLabel;
         IRCInput* queryInput;
         NickInfoPtr m_nickInfo;
+
+        bool m_initialShow;
 };
 #endif
