@@ -1,6 +1,3 @@
-#ifndef SERVER_H
-#define SERVER_H
-
 /*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -9,12 +6,15 @@
 */
 
 /*
-  server.h  -  Server Class to handle connection to the IRC server
-  begin:     Sun Jan 20 2002
-  copyright: (C) 2002,2003,2004 by Dario Abatianni
-  email:     eisfuchs@tigress.com
+  Copyright (C) 2002 Dario Abatianni <eisfuchs@tigress.com>
+  Copyright (C) 2005 Ismail Donmez <ismail@kde.org>
+  Copyright (C) 2005 Peter Simonsson <psn@linux.se>
+  Copyright (C) 2005 Eike Hein <sho@eikehein.com>
 */
 
+#ifndef SERVER_H
+#define SERVER_H
+
 #include <qtimer.h>
 #include <qdict.h>
 
@@ -29,22 +29,23 @@
 #include "channelnick.h"
 #include "inputfilter.h"
 #include "outputfilter.h"
-#include "dcctransfer.h"
 #include "nickinfo.h"
 #include "sslsocket.h"
 #include "serversettings.h"
 #include "servergroupsettings.h"
 
 class Channel;
+class DccTransfer;
 class Query;
 class StatusPanel;
 class Identity;
-class KonversationMainWindow;
 class RawLog;
 class ChannelListPanel;
 class ScriptLauncher;
 class ServerISON;
 class QStrList;
+class ChatWindow;
+class ViewContainer;
 
 using namespace KNetwork;
 
@@ -62,18 +63,20 @@ class Server : public QObject
         /** Constructor used for connecting to a known server.
          *  Read in the prefrences to get all the details about the server.
          */
-        Server(KonversationMainWindow* mainWindow, int serverGroupId, bool clearQuickServerList = true);
+        Server(ViewContainer* viewContainer, int serverGroupId, bool clearQuickServerList = true);
 
         /** Constructor used for a 'fast connect' to a server.
          *  The details are passed in.  Used for example when the user does "/server irc.somewhere.net"
          */
-        Server(KonversationMainWindow* mainWindow,const QString& hostName,const QString& port,
-            const QString& channel,const QString& nick, QString password, const bool& useSSL=FALSE);
+        Server(ViewContainer* viewContainer, const QString& hostName,const QString& port,
+            const QString& channel,const QString& nick, QString password, const bool& useSSL=false);
         ~Server();
 
         QString getServerName() const;
         QString getServerGroup() const;
 
+        void resetCurrentServerIndex() { m_currentServerIndex = 0; }
+
         Konversation::ServerGroupSettingsPtr serverGroupSettings() const { return m_serverGroup; }
 
         IdentityPtr getIdentity() const;
@@ -100,14 +103,12 @@ class Server : public QObject
         QString getAutoJoinChannelKey() const;
         void setAutoJoinChannelKey(const QString &key);
 
-        void setDeliberateQuit(bool on);
-        bool getDeliberateQuit() const;
-
         QString getNextNickname();
 
         void setIrcName(const QString &newIrcName);
         QString getIrcName() const;
 
+        void resetNickList(const QString& channelName);
         void addPendingNickList(const QString& channelName,const QStringList& nickList);
         void addHostmaskToNick(const QString &sourceNick, const QString &sourceHostmask);
         Channel* nickJoinsChannel(const QString &channelName, const QString &nickname, const QString &hostmask);
@@ -115,7 +116,6 @@ class Server : public QObject
         Channel* removeNickFromChannel(const QString &channelName, const QString &nickname, const QString &reason, bool quit=false);
         void nickWasKickedFromChannel(const QString &channelName, const QString &nickname, const QString &kicker, const QString &reason);
         void removeNickFromServer(const QString &nickname, const QString &reason);
-        void noMorePendingNicks(const QString& channel);
 
         void setChannelTypes(const QString &types);
         QString getChannelTypes() const;
@@ -139,7 +139,9 @@ class Server : public QObject
         void appendServerMessageToChannel(const QString& channel, const QString& type, const QString& message);
         void appendCommandMessageToChannel(const QString& channel, const QString& command, const QString& message);
         void appendStatusMessage(const QString& type,const QString& message);
-        void appendMessageToFrontmost(const QString& type,const QString& message);
+        void appendMessageToFrontmost(const QString& type,const QString& message, bool parseURL = true);
+
+        int getPreLength(QString command, QString dest);
 
         void dcopRaw(const QString& command);
         void dcopSay(const QString& target,const QString& command);
@@ -153,12 +155,14 @@ class Server : public QObject
         void updateChannelModeWidgets(const QString& channel, char mode, const QString& parameter);
 
         Channel* getChannelByName(const QString& name);
-        class Query* getQueryByName(const QString& name);
+        Query* getQueryByName(const QString& name);
         QString parseWildcards(const QString& toParse, const QString& nickname, const QString& channelName, const QString &channelKey, const QStringList &nickList, const QString& parameter);
         QString parseWildcards(const QString& toParse, const QString& nickname, const QString& channelName, const QString &channelKey, const QString& nick, const QString& parameter);
 
         QString getAutoJoinCommand() const;
 
+        void autoCommandsAndChannels();
+
         void sendURIs(const QStrList& uris, const QString& nick);
 
         void notifyAction(const QString& nick);
@@ -290,10 +294,7 @@ class Server : public QObject
          *  return false.
          */
         bool isAway() const;
-        /** Put the server in autoaway.  This means that when there is mouse activity, we will set to available again
-         *  @see isAway
-         */
-        void setAutoAway();
+        void setAwayReason(const QString& reason);
 
         void emitChannelNickChanged(const ChannelNickPtr channelNick);
         void emitNickInfoChanged(const NickInfoPtr nickInfo);
@@ -314,7 +315,7 @@ class Server : public QObject
         QStringList getISONList();
         QString getISONListString();
 
-        KonversationMainWindow* getMainWindow() const;
+        ViewContainer* getViewContainer() const;
 
         /** Adds a nickname to the joinedChannels list.
          *  Creates new NickInfo if necessary.
@@ -333,8 +334,8 @@ class Server : public QObject
         void registerWithServices();
 
         // Blowfish stuff
-        QCString getKeyForRecepient(const QString& recepient) const;
-        void setKeyForRecepient(const QString& recepient, const QCString& key);
+        QCString getKeyForRecipient(const QString& recipient) const;
+        void setKeyForRecipient(const QString& recipient, const QCString& key);
 
         bool identifyMsg() const { return m_identifyMsg; }
 
@@ -347,9 +348,6 @@ class Server : public QObject
         void resetLag();
                                                   /// Will be emitted when new 303 came in
         void nicksNowOnline(Server* server,const QStringList& list,bool changed);
-        void addDccPanel();                       /// will be connected to MainWindow::addDccPanel()
-        void addKonsolePanel();                   /// will be connected to MainWindow::addKonsolePanel()
-        void closeDccPanel();                     /// will be connected to MainWindow::closeDccPanel()
         void deleted(Server* myself);             /// will be connected to KonversationApplication::removeServer()
         void awayState(bool away);                /// will be connected to any user input panel;
         void multiServerCommand(const QString& command, const QString& parameter);
@@ -380,6 +378,10 @@ class Server : public QObject
 
         void connectionChangedState(Server* server, Server::State state);
 
+        void showView(ChatWindow* view);
+        void addDccPanel();
+        void addDccChat(const QString& myNick,const QString& nick,const QString& numericalIp,const QStringList& arguments,bool listen);
+
     public slots:
         void lookupFinished();
         void connectToIRCServer();
@@ -392,19 +394,16 @@ class Server : public QObject
          *  @param weinitiated This is whether we initiated this - did we do /query, or somebody else sending us a message.
          *  @return A pointer to a new or already-existing query.  Guaranteed to be non-null
          */
-        class Query *addQuery(const NickInfoPtr & nickInfo, bool weinitiated);
+        Query *addQuery(const NickInfoPtr & nickInfo, bool weinitiated);
         void closeQuery(const QString &name);
         void closeChannel(const QString &name);
         void quitServer();
-        void requestKonsolePanel();
-        void requestDccPanel();
         void requestDccChat(const QString& nickname);
-        void requestCloseDccPanel();
         void requestBan(const QStringList& users,const QString& channel,const QString& option);
         void requestUnban(const QString& mask,const QString& channel);
 
         void addDccSend(const QString &recipient,KURL fileURL, const QString &altFileName = QString::null, uint fileSize = 0);
-        void removeQuery(class Query *query);
+        void removeQuery(Query *query);
         void startNotifyTimer(int msec=0);
         void sendJoinCommand(const QString& channelName, const QString& password = QString::null);
         void requestChannelList();
@@ -429,7 +428,12 @@ class Server : public QObject
         void notifyTimeout();
 
         void enableIdentifyMsg(bool enabled);
-	bool identifyMsgEnabled();
+        bool identifyMsgEnabled();
+        void addBan(const QString &channel, const QString &ban);
+        void removeBan(const QString &channel, const QString &ban);
+
+        /// Called when we received a PONG from the server
+        void pongReceived();
 
     protected slots:
 
@@ -450,7 +454,6 @@ class Server : public QObject
          * @param reason The reason why this failed.  This is already translated, ready to show the user.
          */
         void sslError(QString reason);
-        void notifyCheckTimeout();
         void connectionEstablished(const QString& ownHost);
         void notifyResponse(const QString& nicksOnline);
         void addDccGet(const QString& sourceNick,const QStringList& dccArguments);
@@ -463,9 +466,9 @@ class Server : public QObject
         void resumeDccSendTransfer(const QString& sourceNick,const QStringList& dccArguments);
         void dccSendRequest(const QString& recipient,const QString& fileName,const QString& address,const QString& port,unsigned long size);
         void dccResumeGetRequest(const QString& sender,const QString& fileName,const QString& port,KIO::filesize_t startAt);
-        void dccGetDone(const QString& fileName, DccTransfer::DccStatus status, const QString &errorMessage);
-        void dccSendDone(const QString& fileName, DccTransfer::DccStatus status, const QString &errorMessage);
-        void dccStatusChanged(const DccTransfer* item);
+        void dccGetDone(const DccTransfer* item);
+        void dccSendDone(const DccTransfer* item);
+        void dccStatusChanged(const DccTransfer* item, int newStatus, int oldStatus);
         void away();
         void unAway();
         void scriptNotFound(const QString& name);
@@ -478,12 +481,17 @@ class Server : public QObject
         void gotOwnResolvedHostByWelcome(KResolverResults res);
         void gotOwnResolvedHostByUserhost(KResolverResults res);
 
+        /// Send a PING to the server so we can meassure the lag
+        void sendPing();
+        /// Updates GUI when the lag gets high
+        void updateLongPongLag();
+
     protected:
         // constants
         static const int BUFFER_LEN=513;
 
         /// Initialize the class
-        void init(KonversationMainWindow* mainWindow, const QString& nick, const QString& channel);
+        void init(ViewContainer* viewContainer, const QString& nick, const QString& channel);
 
         /// Initialize the timers
         void initTimers();
@@ -491,11 +499,10 @@ class Server : public QObject
         /// Connect to the signals used in this class.
         void connectSignals();
 
-        void setMainWindow(KonversationMainWindow* newMainWindow);
-
-        void startNotifyCheckTimer();
+        void setViewContainer(ViewContainer* newViewContainer);
 
         void autoRejoinChannels();
+        void updateAutoJoin(const QString& channel=QString::null);
 
         /** Adds a nickname to the unjoinedChannels list.
          *  Creates new NickInfo if necessary.
@@ -568,16 +575,15 @@ class Server : public QObject
         QString channelPrefixes;                  // prefixes that indicate channel names. defaults to RFC1459 "#&"
 
         bool autoJoin;
-        bool autoRejoin;
-        bool autoReconnect;
+        bool quickConnect;
         bool deliberateQuit;
-
-        QStringList connectCommands;
+        bool keepViewsOpenAfterQuit;
+        bool reconnectAfterQuit;
 
         QString autoJoinChannel;
         QString autoJoinChannelKey;
 
-        KonversationMainWindow* mainWindow;
+        ViewContainer* m_viewContainerPtr;
 
         KNetwork::KStreamSocket* m_socket;
         bool         m_tryReconnect;
@@ -590,8 +596,6 @@ class Server : public QObject
         int timerInterval;                        // flood protection
 
         QTimer notifyTimer;
-        QTimer notifyCheckTimer;                  // Checks if the ISON reply needs too long
-        QTime notifySent;
         QStringList notifyCache;                  // List of users found with ISON
         int checkTime;                            // Time elapsed while waiting for server 303 response
         int currentLag;
@@ -617,7 +621,8 @@ class Server : public QObject
         ChannelListPanel* channelListPanel;
 
         bool m_isAway;
-        bool m_isAutoAway;                        ///Note that this may be true, but m_isAway is false, if we go auto-away when disconnected.
+        QString m_awayReason;
+
         bool alreadyConnected;
         bool rejoinChannels;
         bool sendUnlocked;
@@ -665,5 +670,12 @@ class Server : public QObject
 
         /// Used to lock incomingTimer while processing message.
         bool m_processingIncoming;
+
+        /// Meassures the lag between PING and PONG
+        QTime m_lagTime;
+        /// Updates the gui when the lag gets too high
+        QTimer m_pingResponseTimer;
+
+        bool m_autoIdentifyLock;
 };
 #endif
