@@ -39,6 +39,8 @@ class StatusPanel : public ChatWindow
         StatusPanel(QWidget* parent);
         ~StatusPanel();
 
+        virtual void setName(const QString& newName);
+
         virtual QString getTextInLine();
         virtual bool closeYourself();
         virtual bool canBeFrontView();
@@ -47,13 +49,13 @@ class StatusPanel : public ChatWindow
         virtual void setChannelEncoding(const QString& encoding);
         virtual QString getChannelEncoding();
         virtual QString getChannelEncodingDefaultDesc();
-        virtual void setName(const QString& newName) { ChatWindow::setName(newName); }
         virtual void emitUpdateInfo();
 
         virtual void setIdentity(const Identity *newIdentity);
 
-        virtual bool areIRCColorsSupported() {return true; }
-        virtual bool isInsertCharacterSupported() { return true; }
+        virtual bool isInsertSupported() { return true; }
+
+        virtual void setNotificationsEnabled(bool enable);
 
     signals:
         void sendFile();
@@ -65,8 +67,7 @@ class StatusPanel : public ChatWindow
         void updateAppearance();
         virtual void appendInputText(const QString&);
         void updateName();
-        virtual void lostFocus();
- 
+
     protected slots:
         void sendFileMenu();
         void statusTextEntered();
