@@ -6,12 +6,15 @@
 */
 
 /*
-  copyright: (C) 2004 Peter Simonsson <psn@linux.se>
+  Copyright (C) 2004 Peter Simonsson <psn@linux.se>
+  Copyright (C) 2006 Eike Hein <sho@eikehein.com>
 */
+
 #include "topiclabel.h"
 
 #include <qsimplerichtext.h>
 #include <qtooltip.h>
+#include <qclipboard.h>
 
 #include <krun.h>
 #include <kprocess.h>
@@ -20,6 +23,9 @@
 #include <kstringhandler.h>
 #include <kglobal.h>
 #include <kdebug.h>
+#include <kpopupmenu.h>
+#include <kiconloader.h>
+#include <kbookmarkmanager.h>
 
 #include "konversationapplication.h"
 #include "server.h"
@@ -35,9 +41,18 @@ namespace Konversation
         : KActiveLabel(parent, name)
     {
         setWrapPolicy(QTextEdit::AtWordOrDocumentBoundary);
-        mousePressed=false;
         setFocusPolicy(QWidget::ClickFocus);
 
+        m_isOnChannel = false;
+        m_copyUrlMenu = false;
+        mousePressed=false;
+
+        m_popup = new QPopupMenu(this,"topiclabel_context_menu");
+        m_popup->insertItem(SmallIconSet("editcopy"),i18n("&Copy"),Copy);
+        m_popup->insertItem(i18n("Select All"),SelectAll);
+
+        setupChannelPopupMenu();
+
         connect(this, SIGNAL(highlighted(const QString&)), this, SLOT(highlightedSlot(const QString&)));
     }
 
@@ -47,12 +62,14 @@ namespace Konversation
 
     QSize TopicLabel::minimumSizeHint() const
     {
-        return QSize(0, 0);
+        int minHeight = fontMetrics().lineSpacing() + fontMetrics().descent();
+        return QSize(0, minHeight);
     }
 
     QSize TopicLabel::sizeHint() const
     {
-        return QSize(0, 0);
+        int minHeight = fontMetrics().lineSpacing() + fontMetrics().descent();
+        return QSize(0, minHeight);
     }
 
     void TopicLabel::setServer(Server* server)
@@ -65,7 +82,8 @@ namespace Konversation
         if (e->button()==QMouseEvent::LeftButton)
         {
             pressPosition=e->pos();
-            urlToDrag = anchorAt(pressPosition);
+            // Hack to counter the fact that we're given an decoded url
+            urlToDrag = KURL::fromPathOrURL(anchorAt(pressPosition)).url();
             if (!urlToDrag.isNull())
             {
                 mousePressed=true;
@@ -91,7 +109,7 @@ namespace Konversation
         {
             mousePressed=false;
             removeSelection();
-            KURL ux(urlToDrag);
+            KURL ux = KURL::fromPathOrURL(urlToDrag);
             if (urlToDrag.startsWith("##")) ux=QString("irc://%1:%2/%3").arg(m_server->getServerName()).
                     arg(m_server->getPort()).arg(urlToDrag.mid(2));
             KURLDrag* u=new KURLDrag(ux,viewport());
@@ -100,11 +118,17 @@ namespace Konversation
         KActiveLabel::contentsMouseMoveEvent(e);
     }
 
+    void TopicLabel::leaveEvent(QEvent*)
+    {
+       emit clearStatusBarTempText();
+       m_lastStatusText = QString::null;
+    }
+
     void TopicLabel::openLink(const QString& link)
     {
         if (!link.isEmpty())
         {
-            if(link.startsWith("#"))              // channel link
+            if (link.startsWith("#") && m_server && m_server->isConnected())
             {
                 QString channel(link);
                 channel.replace("##","#");
@@ -113,12 +137,12 @@ namespace Konversation
             // Always use KDE default mailer.
             else if (!Preferences::useCustomBrowser() || link.lower().startsWith("mailto:"))
             {
-                new KRun(KURL(link));
+                new KRun(KURL::fromPathOrURL(link));
             }
             else
             {
                 QString cmd = Preferences::webBrowserCmd();
-                cmd.replace("%u",link);
+                cmd.replace("%u",KURL::fromPathOrURL(link).url());
                 KProcess *proc = new KProcess;
                 QStringList cmdAndArgs = KShell::splitArgs(cmd);
                 *proc << cmdAndArgs;
@@ -132,6 +156,74 @@ namespace Konversation
         }
     }
 
+    void TopicLabel::contentsContextMenuEvent(QContextMenuEvent* ev)
+    {
+        bool block = contextMenu(ev);
+
+        if(!block)
+        {
+            KActiveLabel::contentsContextMenuEvent(ev);
+        }
+    }
+
+    bool TopicLabel::contextMenu(QContextMenuEvent* ce)
+    {
+        if (m_isOnChannel)
+        {
+            m_channelPopup->exec(ce->globalPos());
+            m_isOnChannel = false;
+        }
+        else
+        {
+            m_popup->setItemEnabled(Copy,(hasSelectedText()));
+
+            int r = m_popup->exec(ce->globalPos());
+
+            switch(r)
+            {
+                case -1:
+                    // dummy. -1 means, no entry selected. we don't want -1to go in default, so
+                    // we catch it here
+                    break;
+                case Copy:
+                    copy();
+                    break;
+                case CopyUrl:
+                {
+                    QClipboard *cb = KApplication::kApplication()->clipboard();
+                    cb->setText(m_urlToCopy,QClipboard::Selection);
+                    cb->setText(m_urlToCopy,QClipboard::Clipboard);
+                    break;
+                }
+                case SelectAll:
+                    selectAll();
+                    break;
+                case Bookmark:
+                {
+                    KBookmarkManager* bm = KBookmarkManager::userBookmarksManager();
+                    KBookmarkGroup bg = bm->addBookmarkDialog(m_urlToCopy, QString::null);
+                    bm->save();
+                    bm->emitChanged(bg);
+                    break;
+                }
+                default:
+                    break;
+            }
+        }
+        return true;
+    }
+
+    void TopicLabel::setupChannelPopupMenu()
+    {
+        m_channelPopup = new KPopupMenu(this,"channel_context_menu");
+        m_channelPopupId = m_channelPopup->insertTitle(m_currentChannel);
+        m_channelPopup->insertItem(i18n("&Join"),Konversation::Join);
+        m_channelPopup->insertItem(i18n("Get &user list"),Konversation::Names);
+        m_channelPopup->insertItem(i18n("Get &topic"),Konversation::Topic);
+
+        connect(m_channelPopup, SIGNAL(activated(int)), this, SIGNAL(popupCommand(int)));
+    }
+
     void TopicLabel::setText(const QString& text)
     {
         m_fullText = text;
@@ -142,9 +234,9 @@ namespace Konversation
     {
         QFontMetrics fm(currentFont());
         QString text = m_fullText;
-        text.replace("&", "&amp;").
-            replace("<", "&lt;").
-            replace(">", "&gt;");
+        // text.replace("&", "&amp;"). Not needed as we do it in tagURLs
+        text.replace("<", "\x0blt;"). // tagUrls will replace \x0b with &
+            replace(">", "\x0bgt;");
         text = tagURLs(text, "", false);
 
         QToolTip::remove(this);
@@ -210,40 +302,68 @@ namespace Konversation
         return richText.widthUsed();
     }
 
-    void TopicLabel::highlightedSlot(const QString& link)
+    void TopicLabel::highlightedSlot(const QString& _link)
     {
-        if(link == m_lastStatusText && !link.isEmpty()) {
+        QString link = KURL::fromPathOrURL(_link).url();
         //we just saw this a second ago.  no need to reemit.
+        if (link == m_lastStatusText && !link.isEmpty())
             return;
-        }
 
         // remember current URL to overcome link clicking problems in QTextBrowser
-        m_highlightedURL=link;
+        m_highlightedURL = link;
 
-        if(link.isEmpty()) {
-            if(!m_lastStatusText.isEmpty()) {
-                emit clearStatusText();
+        if (link.isEmpty())
+        {
+            if (!m_lastStatusText.isEmpty())
+            {
+                emit clearStatusBarTempText();
                 m_lastStatusText = QString::null;
             }
         } else {
             m_lastStatusText = link;
         }
 
-        if (!link.isEmpty())
+        if (!link.startsWith("#"))
         {
-            if(link.startsWith("#"))              // channel link
-            {
-                QString channel(link);
-                channel.replace("##","#");
-                emit actionStatusText( i18n("Join the channel %1").arg(channel));
+            m_isOnChannel = false;
+
+            if (!link.isEmpty()) {
+                //link therefore != m_lastStatusText  so emit with this new text
+                emit setStatusBarTempText(link);
             }
-            else
+            if (link.isEmpty() && m_copyUrlMenu)
             {
-                emit actionStatusText(link);
+                m_popup->removeItem(CopyUrl);
+                m_popup->removeItem(Bookmark);
+                m_copyUrlMenu = false;
+            }
+            else if(!link.isEmpty() && !m_copyUrlMenu)
+            {
+                m_popup->insertItem(i18n("Copy URL to Clipboard"),CopyUrl,1);
+                m_popup->insertItem(i18n("Add to Bookmarks"),Bookmark,2);
+                m_copyUrlMenu = true;
+                m_urlToCopy = link;
             }
         }
-    }
+        else if (link.startsWith("##"))
+        {
+            m_currentChannel = link.mid(1);
 
+            emit currentChannelChanged(m_currentChannel);
+
+            QString prettyId = m_currentChannel;
+
+            if (prettyId.length()>15)
+            {
+                prettyId.truncate(15);
+                prettyId.append("...");
+            }
+
+            m_channelPopup->changeTitle(m_channelPopupId,prettyId);
+            m_isOnChannel = true;
+            emit setStatusBarTempText(i18n("Join the channel %1").arg(m_currentChannel));
+        }
+    }
 }
 
 #include "topiclabel.moc"
